@@ -402,12 +402,10 @@ If an unsupported TYPE is requested, the function will return NIL.
 (defun find-definition-source (object)
   (typecase object
     ((or sb-pcl::condition-class sb-pcl::structure-class)
-     (let ((classoid (sb-impl::find-classoid (class-name object))))
+     (let ((classoid (sb-pcl::class-classoid object)))
        (when classoid
-         (let ((layout (sb-impl::classoid-layout classoid)))
-           (when layout
-             (translate-source-location
-              (sb-kernel::layout-source-location layout)))))))
+         (translate-source-location
+          (sb-kernel::classoid-source-location classoid)))))
     (method-combination
      (car
       (find-definition-sources-by-name
@@ -947,6 +945,14 @@ Experimental: interface subject to change."
          (call (%simple-fun-type object))
          (call (%simple-fun-info object)))
         (symbol
+         ;; We use :override here because (apparently) the intent is
+         ;; to avoid calling FUNCTION on the SYMBOL-PACKAGE
+         ;; when SIMPLE is NIL (the default). And we skip SYMBOL-EXTRA for
+         ;; the same reason that we don't call FUNCTION on SYMBOL-INFO
+         ;; (logically it's "system" data, not for user consumption).
+         ;; Frankly this entire function is a confusing mishmash that is not
+         ;; accurate for computing a true graph of objects starting from a
+         ;; certain point, given all the special cases that it implements.
          :override
          (when ext
            (dolist (thread (sb-thread:list-all-threads))
@@ -980,11 +986,9 @@ Experimental: interface subject to change."
   (+ (primitive-object-size object)
      (typecase object
        (sb-mop:funcallable-standard-object
-        (primitive-object-size
-         (sb-pcl::standard-funcallable-instance-clos-slots object)))
+        (primitive-object-size (sb-pcl::fsc-instance-slots object)))
        (standard-object
-        (primitive-object-size
-         (sb-pcl::standard-instance-slots object)))
+        (primitive-object-size (sb-pcl::std-instance-slots object)))
        (t 0))))
 
 ;;; Print a distribution of object sizes in SPACE.
