@@ -31,23 +31,35 @@
 
 ;;;; basic LAYOUT stuff
 
-;;; a list of conses, initialized by genesis
+;;; a vector of conses, initialized by genesis
 ;;;
 ;;; In each cons, the car is the symbol naming the layout, and the
 ;;; cdr is the layout itself.
 (defvar *!initial-layouts*)
 
+;;; a generator for random values suitable for the CLOS-HASH slots of
+;;; LAYOUTs. We use our own RANDOM-STATE here because we'd like
+;;; pseudo-random values to come the same way in the target even when
+;;; we make minor changes to the system, in order to reduce the
+;;; mysteriousness of possible CLOS bugs.
+(define-load-time-global *layout-clos-hash-random-state*
+    (make-random-state))
+
 ;;; a table mapping class names to layouts for classes we have
 ;;; referenced but not yet loaded. This is initialized from an alist
 ;;; created by genesis describing the layouts that genesis created at
 ;;; cold-load time.
-(defvar *forward-referenced-layouts*)
+(define-load-time-global *forward-referenced-layouts*
+    ;; FIXME: why is the test EQUAL and not EQ? Aren't the keys all symbols?
+    (make-hash-table :test 'equal))
 (!cold-init-forms
-  ;; Protected by *WORLD-LOCK*
-  (setq *forward-referenced-layouts* (make-hash-table :test 'equal))
+  ;; *forward-referenced-layouts* is protected by *WORLD-LOCK*
+  ;; so it does not need a :synchronized option.
   #-sb-xc-host (progn
                  (/show0 "processing *!INITIAL-LAYOUTS*")
-                 (dolist (x *!initial-layouts*)
+                 (setq *forward-referenced-layouts* (make-hash-table :test 'equal))
+                 (setq *layout-clos-hash-random-state* (make-random-state))
+                 (dovector (x *!initial-layouts*)
                    (setf (layout-clos-hash (cdr x)) (random-layout-clos-hash))
                    (setf (gethash (car x) *forward-referenced-layouts*)
                          (cdr x)))
@@ -55,9 +67,6 @@
 
 ;;; The LAYOUT structure itself is defined in 'early-classoid.lisp'
 
-(declaim (inline layout-for-std-class-p))
-(defun layout-for-std-class-p (x) (not (zerop (layout-%for-std-class-b x))))
-
 (defmethod print-object ((layout layout) stream)
   (print-unreadable-object (layout stream :type t :identity t)
     (format stream
@@ -71,12 +80,6 @@
 
 ;;;; support for the hash values used by CLOS when working with LAYOUTs
 
-;;; a generator for random values suitable for the CLOS-HASH slots of
-;;; LAYOUTs. We use our own RANDOM-STATE here because we'd like
-;;; pseudo-random values to come the same way in the target even when
-;;; we make minor changes to the system, in order to reduce the
-;;; mysteriousness of possible CLOS bugs.
-(defvar *layout-clos-hash-random-state*)
 (defun random-layout-clos-hash ()
   ;; FIXME: I'm not sure why this expression is (1+ (RANDOM FOO)),
   ;; returning a strictly positive value. I copied it verbatim from
@@ -89,10 +92,7 @@
   ;; an explanation is provided in Kiczales and Rodriguez, "Efficient
   ;; Method Dispatch in PCL", 1990.  -- CSR, 2005-11-30
   (1+ (random (1- layout-clos-hash-limit)
-              (if (boundp '*layout-clos-hash-random-state*)
-                  *layout-clos-hash-random-state*
-                  (setf *layout-clos-hash-random-state*
-                        (make-random-state))))))
+              *layout-clos-hash-random-state*)))
 
 ;;; If we can't find any existing layout, then we create a new one
 ;;; storing it in *FORWARD-REFERENCED-LAYOUTS*. In classic CMU CL, we
@@ -112,9 +112,11 @@
     (with-world-lock ()
       (let ((classoid (find-classoid name nil)))
         (or (and classoid (classoid-layout classoid))
-            (gethash name table)
-            (setf (gethash name table)
-                  (make-layout :classoid (or classoid (make-undefined-classoid name)))))))))
+            (values (ensure-gethash name table
+                                    (make-layout
+                                     :classoid
+                                     (or classoid
+                                         (make-undefined-classoid name))))))))))
 
 ;;; If LAYOUT is uninitialized, initialize it with CLASSOID, LENGTH,
 ;;; INHERITS, DEPTHOID, and BITMAP.
@@ -125,14 +127,15 @@
 ;;; preexisting class slot value is OK, and if it's not initialized,
 ;;; its class slot value is set to an UNDEFINED-CLASS. -- FIXME: This
 ;;; is no longer true, :UNINITIALIZED used instead.
-(declaim (ftype (function (layout classoid index simple-vector layout-depthoid layout-bitmap)
-                          layout)
-                %init-or-check-layout))
-(defun %init-or-check-layout (layout classoid length inherits depthoid bitmap)
+(declaim (ftype (function (layout classoid index fixnum simple-vector
+                           layout-depthoid layout-bitmap)
+                          layout) %init-or-check-layout))
+(defun %init-or-check-layout (layout classoid length flags inherits depthoid bitmap)
   (cond ((eq (layout-invalid layout) :uninitialized)
          ;; There was no layout before, we just created one which
          ;; we'll now initialize with our information.
          (setf (layout-length layout) length
+               (layout-%flags layout) flags
                (layout-inherits layout) inherits
                (layout-depthoid layout) depthoid
                (layout-bitmap layout) bitmap
@@ -163,7 +166,15 @@
   (when (layout-invalid layout)
     (sb!c::compiler-error "can't dump reference to obsolete class: ~S"
                           (layout-classoid layout)))
-  (let ((name (classoid-name (layout-classoid layout))))
+  (let* ((classoid (layout-classoid layout))
+         (name (classoid-name classoid)))
+    (aver (= (layout-%flags layout)
+             (typecase classoid
+               (structure-classoid +structure-layout-flag+)
+               (condition-classoid +condition-layout-flag+)
+               (undefined-classoid
+                (bug "xc MAKE-LOAD-FORM on undefined layout"))
+               (t 0))))
     (unless name
       (sb!c::compiler-error "can't dump anonymous LAYOUT: ~S" layout))
     ;; Since LAYOUT refers to a class which refers back to the LAYOUT,
@@ -178,6 +189,7 @@
      `(%init-or-check-layout ',layout
                              ',(layout-classoid layout)
                              ',(layout-length layout)
+                             ',(layout-%flags layout)
                              ',(layout-inherits layout)
                              ',(layout-depthoid layout)
                              ',(layout-bitmap layout)))))
@@ -270,10 +282,10 @@ between the ~A definition and the ~A definition"
 ;;; Used by the loader to forward-reference layouts for classes whose
 ;;; definitions may not have been loaded yet. This allows type tests
 ;;; to be loaded when the type definition hasn't been loaded yet.
-(declaim (ftype (function (symbol index simple-vector layout-depthoid layout-bitmap)
+(declaim (ftype (function (symbol index fixnum simple-vector layout-depthoid layout-bitmap)
                           layout)
                 find-and-init-or-check-layout))
-(defun find-and-init-or-check-layout (name length inherits depthoid bitmap)
+(defun find-and-init-or-check-layout (name length flags inherits depthoid bitmap)
   (truly-the ; avoid an "assertion too complex to check" optimizer note
    (values layout &optional)
    (with-world-lock ()
@@ -282,6 +294,7 @@ between the ~A definition and the ~A definition"
                              (or (find-classoid name nil)
                                  (layout-classoid layout))
                              length
+                             flags
                              inherits
                              depthoid
                              bitmap)))))
@@ -418,17 +431,9 @@ between the ~A definition and the ~A definition"
   (let ((obj-info (make-hash-table :size (length objects)))
         (free-objs nil)
         (result nil))
-    (dolist (constraint constraints)
-      (let ((obj1 (car constraint))
-            (obj2 (cdr constraint)))
-        (let ((info2 (gethash obj2 obj-info)))
-          (if info2
-              (incf (first info2))
-              (setf (gethash obj2 obj-info) (list 1))))
-        (let ((info1 (gethash obj1 obj-info)))
-          (if info1
-              (push obj2 (rest info1))
-              (setf (gethash obj1 obj-info) (list 0 obj2))))))
+    (loop for (obj1 . obj2) in constraints do
+       (incf (first (ensure-gethash obj2 obj-info (list 0))))
+       (push obj2 (rest (ensure-gethash obj1 obj-info (list 0)))))
     (dolist (obj objects)
       (let ((info (gethash obj obj-info)))
         (when (or (not info) (zerop (first info)))
@@ -493,9 +498,11 @@ between the ~A definition and the ~A definition"
 ;;; classes. Non-structure "typed" defstructs are a special case, and
 ;;; don't have a corresponding class.
 (def!struct (structure-classoid (:include classoid)
-                                (:constructor %make-structure-classoid)))
+                                (:copier nil)
+                                (:constructor %make-structure-classoid
+                                    (hash-value name))))
 (defun make-structure-classoid (&key name)
-  (mark-ctype-interned (%make-structure-classoid :name name)))
+  (%make-structure-classoid (interned-type-hash name) name))
 
 ;;;; classoid namespace
 
@@ -544,7 +551,7 @@ between the ~A definition and the ~A definition"
                      ~/sb!impl:print-type-specifier/." name))
             (:defined
              (warn "redefining DEFTYPE type to be a class: ~
-                    ~/sb!impl::print-symbol-with-prefix/" name)
+                    ~/sb!ext:print-symbol-with-prefix/" name)
              (clear-info :type :expander name)
              (clear-info :type :source-location name)))
 
@@ -967,7 +974,7 @@ between the ~A definition and the ~A definition"
      (sequence
       :translation (or cons (member nil) vector extended-sequence)
       :state :read-only
-      :depth 2)
+      :depth 1)
      (vector
       :translation vector :codes (#.sb!vm:complex-vector-widetag)
       :direct-superclasses (array sequence)
@@ -987,151 +994,6 @@ between the ~A definition and the ~A definition"
       :inherits (bit-vector vector simple-array
                  array sequence)
       :prototype-form (make-array 0 :element-type 'bit))
-     (simple-array-unsigned-byte-2
-      :translation (simple-array (unsigned-byte 2) (*))
-      :codes (#.sb!vm:simple-array-unsigned-byte-2-widetag)
-      :direct-superclasses (vector simple-array)
-      :inherits (vector simple-array array sequence)
-      :prototype-form (make-array 0 :element-type '(unsigned-byte 2)))
-     (simple-array-unsigned-byte-4
-      :translation (simple-array (unsigned-byte 4) (*))
-      :codes (#.sb!vm:simple-array-unsigned-byte-4-widetag)
-      :direct-superclasses (vector simple-array)
-      :inherits (vector simple-array array sequence)
-      :prototype-form (make-array 0 :element-type '(unsigned-byte 4)))
-     (simple-array-unsigned-byte-7
-      :translation (simple-array (unsigned-byte 7) (*))
-      :codes (#.sb!vm:simple-array-unsigned-byte-7-widetag)
-      :direct-superclasses (vector simple-array)
-      :inherits (vector simple-array array sequence)
-      :prototype-form (make-array 0 :element-type '(unsigned-byte 7)))
-     (simple-array-unsigned-byte-8
-      :translation (simple-array (unsigned-byte 8) (*))
-      :codes (#.sb!vm:simple-array-unsigned-byte-8-widetag)
-      :direct-superclasses (vector simple-array)
-      :inherits (vector simple-array array sequence)
-      :prototype-form (make-array 0 :element-type '(unsigned-byte 8)))
-     (simple-array-unsigned-byte-15
-      :translation (simple-array (unsigned-byte 15) (*))
-      :codes (#.sb!vm:simple-array-unsigned-byte-15-widetag)
-      :direct-superclasses (vector simple-array)
-      :inherits (vector simple-array array sequence)
-      :prototype-form (make-array 0 :element-type '(unsigned-byte 15)))
-     (simple-array-unsigned-byte-16
-      :translation (simple-array (unsigned-byte 16) (*))
-      :codes (#.sb!vm:simple-array-unsigned-byte-16-widetag)
-      :direct-superclasses (vector simple-array)
-      :inherits (vector simple-array array sequence)
-      :prototype-form (make-array 0 :element-type '(unsigned-byte 16)))
-
-     (simple-array-unsigned-fixnum
-      :translation (simple-array (unsigned-byte #.sb!vm:n-positive-fixnum-bits) (*))
-      :codes (#.sb!vm:simple-array-unsigned-fixnum-widetag)
-      :direct-superclasses (vector simple-array)
-      :inherits (vector simple-array array sequence)
-      :prototype-form (make-array 0
-                       :element-type '(unsigned-byte #.sb!vm:n-positive-fixnum-bits)))
-
-     (simple-array-unsigned-byte-31
-      :translation (simple-array (unsigned-byte 31) (*))
-      :codes (#.sb!vm:simple-array-unsigned-byte-31-widetag)
-      :direct-superclasses (vector simple-array)
-      :inherits (vector simple-array array sequence)
-      :prototype-form (make-array 0 :element-type '(unsigned-byte 31)))
-     (simple-array-unsigned-byte-32
-      :translation (simple-array (unsigned-byte 32) (*))
-      :codes (#.sb!vm:simple-array-unsigned-byte-32-widetag)
-      :direct-superclasses (vector simple-array)
-      :inherits (vector simple-array array sequence)
-      :prototype-form (make-array 0 :element-type '(unsigned-byte 32)))
-     #!+64-bit
-     (simple-array-unsigned-byte-63
-      :translation (simple-array (unsigned-byte 63) (*))
-      :codes (#.sb!vm:simple-array-unsigned-byte-63-widetag)
-      :direct-superclasses (vector simple-array)
-      :inherits (vector simple-array array sequence)
-      :prototype-form (make-array 0 :element-type '(unsigned-byte 63)))
-     #!+64-bit
-     (simple-array-unsigned-byte-64
-      :translation (simple-array (unsigned-byte 64) (*))
-      :codes (#.sb!vm:simple-array-unsigned-byte-64-widetag)
-      :direct-superclasses (vector simple-array)
-      :inherits (vector simple-array array sequence)
-      :prototype-form (make-array 0 :element-type '(unsigned-byte 64)))
-     (simple-array-signed-byte-8
-      :translation (simple-array (signed-byte 8) (*))
-      :codes (#.sb!vm:simple-array-signed-byte-8-widetag)
-      :direct-superclasses (vector simple-array)
-      :inherits (vector simple-array array sequence)
-      :prototype-form (make-array 0 :element-type '(signed-byte 8)))
-     (simple-array-signed-byte-16
-      :translation (simple-array (signed-byte 16) (*))
-      :codes (#.sb!vm:simple-array-signed-byte-16-widetag)
-      :direct-superclasses (vector simple-array)
-      :inherits (vector simple-array array sequence)
-      :prototype-form (make-array 0 :element-type '(signed-byte 16)))
-
-     (simple-array-fixnum
-      :translation (simple-array (signed-byte #.sb!vm:n-fixnum-bits)
-                    (*))
-      :codes (#.sb!vm:simple-array-fixnum-widetag)
-      :direct-superclasses (vector simple-array)
-      :inherits (vector simple-array array sequence)
-      :prototype-form (make-array 0
-                       :element-type
-                       '(signed-byte #.sb!vm:n-fixnum-bits)))
-
-     (simple-array-signed-byte-32
-      :translation (simple-array (signed-byte 32) (*))
-      :codes (#.sb!vm:simple-array-signed-byte-32-widetag)
-      :direct-superclasses (vector simple-array)
-      :inherits (vector simple-array array sequence)
-      :prototype-form (make-array 0 :element-type '(signed-byte 32)))
-     #!+64-bit
-     (simple-array-signed-byte-64
-      :translation (simple-array (signed-byte 64) (*))
-      :codes (#.sb!vm:simple-array-signed-byte-64-widetag)
-      :direct-superclasses (vector simple-array)
-      :inherits (vector simple-array array sequence)
-      :prototype-form (make-array 0 :element-type '(signed-byte 64)))
-     (simple-array-single-float
-      :translation (simple-array single-float (*))
-      :codes (#.sb!vm:simple-array-single-float-widetag)
-      :direct-superclasses (vector simple-array)
-      :inherits (vector simple-array array sequence)
-      :prototype-form (make-array 0 :element-type 'single-float))
-     (simple-array-double-float
-      :translation (simple-array double-float (*))
-      :codes (#.sb!vm:simple-array-double-float-widetag)
-      :direct-superclasses (vector simple-array)
-      :inherits (vector simple-array array sequence)
-      :prototype-form (make-array 0 :element-type 'double-float))
-     #!+long-float
-     (simple-array-long-float
-      :translation (simple-array long-float (*))
-      :codes (#.sb!vm:simple-array-long-float-widetag)
-      :direct-superclasses (vector simple-array)
-      :inherits (vector simple-array array sequence)
-      :prototype-form (make-array 0 :element-type 'long-float))
-     (simple-array-complex-single-float
-      :translation (simple-array (complex single-float) (*))
-      :codes (#.sb!vm:simple-array-complex-single-float-widetag)
-      :direct-superclasses (vector simple-array)
-      :inherits (vector simple-array array sequence)
-      :prototype-form (make-array 0 :element-type '(complex single-float)))
-     (simple-array-complex-double-float
-      :translation (simple-array (complex double-float) (*))
-      :codes (#.sb!vm:simple-array-complex-double-float-widetag)
-      :direct-superclasses (vector simple-array)
-      :inherits (vector simple-array array sequence)
-      :prototype-form (make-array 0 :element-type '(complex double-float)))
-     #!+long-float
-     (simple-array-complex-long-float
-      :translation (simple-array (complex long-float) (*))
-      :codes (#.sb!vm:simple-array-complex-long-float-widetag)
-      :direct-superclasses (vector simple-array)
-      :inherits (vector simple-array array sequence)
-      :prototype-form (make-array 0 :element-type '(complex long-float)))
      (string
       :translation string
       :direct-superclasses (vector)
@@ -1194,6 +1056,11 @@ between the ~A definition and the ~A definition"
       :inherits (symbol list sequence)
       :direct-superclasses (symbol list)
       :prototype-form 'nil)
+     ;; These last few are strange. STREAM has only T as an ancestor,
+     ;; so you'd think it would be at depth 1. FILE- and STRING-STREAM
+     ;; each have STREAM and T as ancestors, so you'd think they'd be at depth
+     ;; 1 greater than STREAM, instead of 2 greater. But changing any of
+     ;; these to the "obvious" value makes various type checks go wrong.
      (stream
       :state :read-only
       :depth 2)
@@ -1204,7 +1071,21 @@ between the ~A definition and the ~A definition"
      (string-stream
       :state :read-only
       :depth 4
-      :inherits (stream))))
+      :inherits (stream))
+     .
+     #.(loop for x across sb!vm:*specialized-array-element-type-properties*
+             unless (member (sb!vm::saetp-specifier x) '(t character base-char nil bit))
+             collect
+             ;; I'm not sure if it's an accident that there are distinct SB!KERNEL
+             ;; versus SB!VM symbols for the specialized arrays. The former are types
+             ;; in the language, and the latter are primitive object types,
+             ;; but istm they should be designated by the same symbols.
+             `(,(intern (string (sb!vm::saetp-primitive-type-name x)) *package*)
+               :translation (simple-array ,(sb!vm::saetp-specifier x) (*))
+               :codes (,(sb!vm::saetp-typecode x))
+               :direct-superclasses (vector simple-array)
+               :inherits (vector simple-array array sequence)
+               :prototype-form (make-array 0 :element-type ',(sb!vm::saetp-specifier x))))))
 
 ;;; See also src/code/class-init.lisp where we finish setting up the
 ;;; translations for built-in types.
@@ -1237,15 +1118,15 @@ between the ~A definition and the ~A definition"
                     (t
                      (setf (classoid-cell-classoid
                             (find-classoid-cell name :create t))
-                           (mark-ctype-interned
-                            (make-built-in-classoid
+                           (!make-built-in-classoid
+                             :hash-value (interned-type-hash name)
                              :name name
                              :translation (if trans-p :initializing nil)
                              :direct-superclasses
                              (if (eq name t)
                                  nil
                                  (mapcar #'find-classoid
-                                         direct-superclasses)))))))))
+                                         direct-superclasses))))))))
         (setf (info :type :kind name) :primitive)
         (unless trans-p
           (setf (info :type :builtin name) classoid))
@@ -1263,6 +1144,7 @@ between the ~A definition and the ~A definition"
                            -1)))
           (register-layout
            (find-and-init-or-check-layout name
+                                          0
                                           0
                                           inherits-vector
                                           depthoid
@@ -1332,13 +1214,21 @@ between the ~A definition and the ~A definition"
   #-sb-xc-host (/show0 "about to set *BUILT-IN-CLASS-CODES*")
   (setq **built-in-class-codes**
         (let* ((initial-element (classoid-layout (find-classoid 'random-class)))
-               (res (make-array 256 :initial-element initial-element)))
-          (dolist (x *!built-in-classes* res)
+               (table (make-array 256 :initial-element initial-element)))
+          (dolist (x *!built-in-classes*)
             (destructuring-bind (name &key codes &allow-other-keys)
                                 x
               (let ((layout (classoid-layout (find-classoid name))))
                 (dolist (code codes)
-                  (setf (svref res code) layout)))))))
-  #-sb-xc-host (/show0 "done setting *BUILT-IN-CLASS-CODES*"))
+                  (setf (svref table code) layout)))))
+          (loop with layout = (aref table sb!vm:list-pointer-lowtag)
+                for i from sb!vm:list-pointer-lowtag by (* 2 sb!vm:n-word-bytes)
+                below 256
+                do (setf (aref table i) layout))
+          (loop with layout = (aref table sb!vm:even-fixnum-lowtag)
+                for i from sb!vm:even-fixnum-lowtag by (ash 1 sb!vm:n-fixnum-tag-bits)
+                below 256
+                do (setf (aref table i) layout))
+          table)))
 
 (!defun-from-collected-cold-init-forms !classes-cold-init)
