@@ -21,6 +21,7 @@
 ;;; is declared to hold a DEFSTRUCT-DESCRIPTION.
 (def!struct (defstruct-description
              (:conc-name dd-)
+             (:copier nil)
              #-sb-xc-host (:pure t)
              (:constructor make-defstruct-description (null-lexenv-p name)))
   ;; name of the structure
@@ -73,9 +74,13 @@
   ;; NIL if the option was given with no argument.
   (printer-fname nil :type (or cons symbol))
 
-  ;; the value of the :PURE option, or :UNSPECIFIED. This is only
-  ;; meaningful if DD-CLASS-P = T.
-  (pure :unspecified :type (member t nil :unspecified)))
+  ;; the value of the :PURE option, used by cheneygc when purifying.
+  ;; This is true if objects of this class are never modified to
+  ;; contain dynamic pointers in their slots or constant-like
+  ;; substructure (and hence can be copied into read-only space by
+  ;; PURIFY).
+  ;; This is only meaningful if DD-CLASS-P = T.
+  (pure nil :type (member t nil)))
 #!-sb-fluid (declaim (freeze-type defstruct-description))
 (!set-load-form-method defstruct-description (:host :xc :target))
 
@@ -94,6 +99,10 @@
 (def!type layout-clos-hash () `(integer 0 ,layout-clos-hash-limit))
 (declaim (ftype (sfunction () layout-clos-hash) random-layout-clos-hash))
 
+(defconstant +structure-layout-flag+  #b001)
+(defconstant +pcl-object-layout-flag+ #b010)
+(defconstant +condition-layout-flag+  #b100)
+
 ;;; The LAYOUT structure is pointed to by the first cell of instance
 ;;; (or structure) objects. It represents what we need to know for
 ;;; type checking and garbage collection. Whenever a class is
@@ -106,7 +115,10 @@
 ;;; well, since the initialization of layout slots is hardcoded there.
 ;;;
 ;;; FIXME: ...it would be better to automate this, of course...
-(def!struct (layout #-sb-xc-host (:constructor #!+immobile-space nil))
+(def!struct (layout #-sb-xc-host (:constructor #!+immobile-space nil)
+                    (:copier nil))
+  ;; one +something-LAYOUT-FLAG+ bit or none of them
+  (%flags 0 :type fixnum :read-only nil)
   ;; a pseudo-random hash value for use by CLOS.
   (clos-hash (random-layout-clos-hash) :type layout-clos-hash)
   ;; the class that this is a layout for
@@ -141,42 +153,30 @@
   ;; the number of top level descriptor cells in each instance
   ;; For [FUNCALLABLE-]STANDARD-OBJECT instances, this is the slot vector
   ;; length, not the primitive object length.
-  (length 0 :type index)
+  ;; I tried making a structure of this many slots, and the compiler blew up;
+  ;; so it's fair to say this limit is sufficient for practical purposes,
+  ;; Let's be consistent here between the two choices of word size.
+  (length 0 :type (unsigned-byte 28)) ; smaller than SB-INT:INDEX
   ;; If this layout has some kind of compiler meta-info, then this is
   ;; it. If a structure, then we store the DEFSTRUCT-DESCRIPTION here.
   (info nil :type (or null defstruct-description))
-  ;; This is true if objects of this class are never modified to
-  ;; contain dynamic pointers in their slots or constant-like
-  ;; substructure (and hence can be copied into read-only space by
-  ;; PURIFY).
-  ;;
-  ;; This slot is known to the C runtime support code.
-  (pure nil :type (member t nil 0))
   ;; Map of raw slot indices.
   (bitmap +layout-all-tagged+ :type layout-bitmap)
   ;; Per-slot comparator for implementing EQUALP.
   (equalp-tests #() :type simple-vector)
-  ;; Definition location
-  (source-location nil)
   ;; If this layout is for an object of metatype STANDARD-CLASS,
   ;; these are the EFFECTIVE-SLOT-DEFINITION metaobjects.
   (slot-list nil :type list)
   ;; Information about slots in the class to PCL: this provides fast
   ;; access to slot-definitions and locations by name, etc.
   ;; See MAKE-SLOT-TABLE in pcl/slots-boot.lisp for further details.
-  (slot-table #(1 nil) :type simple-vector)
-  ;; True IFF the layout belongs to a standand-instance or a
-  ;; standard-funcallable-instance.
-  ;; Old comment was:
-  ;;   FIXME: If we unify wrappers and layouts this can go away, since
-  ;;   it is only used in SB-PCL::EMIT-FETCH-WRAPPERS, which can then
-  ;;   use INSTANCE-SLOTS-LAYOUT instead (if there is are no slot
-  ;;   layouts, there are no slots for it to pull.)
-  ;; But while that's conceivable, it still seems advantageous to have
-  ;; a single bit that decides whether something is STANDARD-OBJECT.
-  (%for-std-class-b 0 :type bit :read-only t))
+  (slot-table #(1 nil) :type simple-vector))
 (declaim (freeze-type layout))
 
+(declaim (inline layout-for-std-class-p))
+(defun layout-for-std-class-p (x)
+  (logtest (layout-%flags x) +pcl-object-layout-flag+))
+
 #!+(and immobile-space (host-feature sb-xc))
 (macrolet ((def-layout-maker ()
              (let ((slots (dd-slots (find-defstruct-description 'layout))))
@@ -191,6 +191,9 @@
                   (dx-let ((data (vector ,@(mapcar #'dsd-name slots))))
                     (truly-the layout
                      (values (%primitive sb!vm::alloc-immobile-layout data))))))))
+   (assert (<= (* sb!vm:n-word-bytes
+                 (1+ (dd-length (find-defstruct-description 'layout))))
+              sb!vm::layout-align))
   (def-layout-maker))
 
 ;;; The CLASSOID structure is a supertype of all classoid types.  A
@@ -202,6 +205,7 @@
              (:include ctype
                        (class-info (type-class-or-lose 'classoid)))
              (:constructor nil)
+             (:copier nil)
              #-no-ansi-print-object
              (:print-object
               (lambda (class stream)
@@ -228,6 +232,9 @@
   (state nil :type (member nil :read-only :sealed))
   ;; direct superclasses of this class. Always NIL for CLOS classes.
   (direct-superclasses () :type list)
+  ;; Definition location
+  ;; Not used for standard-classoid, because pcl has its own mechanism.
+  (source-location nil)
   ;; representation of all of the subclasses (direct or indirect) of
   ;; this class. This is NIL if no subclasses or not initalized yet;
   ;; otherwise, it's an EQ hash-table mapping CLASSOID objects to the
@@ -239,12 +246,39 @@
   ;; NIL if none assigned yet
   (pcl-class nil))
 
+;;; A helper to make classoid (and named-type) hash values stable.
+;;; For other ctypes, generally improve the randomness of the hash.
+;;; (The host uses at most 21 bits of randomness. See CTYPE-RANDOM)
+#+sb-xc
+(defun !improve-ctype-hash (obj type-class-name)
+  (let ((hash (case type-class-name
+                (named
+                 (interned-type-hash (named-type-name obj) 'named))
+                (classoid
+                 (interned-type-hash (classoid-name obj)))
+                (t
+                 (interned-type-hash))))
+        ;; Preserve the interned-p and type=-optimization bits
+        ;; by affecting only bits under the ctype-hash-mask.
+        ;; Upper 5 hash bits might be an index into SAETP array
+        ;; (if this ctype is exactly a type to which upgrade occurs)
+        (nbits (- (integer-length +ctype-hash-mask+)
+                  +ctype-saetp-index-bits+)))
+    (macrolet ((slot-index ()
+                 (let* ((dd (find-defstruct-description 'ctype))
+                        (dsd (find 'hash-value (dd-slots dd) :key #'dsd-name)))
+                   (dsd-index dsd))))
+      (setf (%instance-ref obj (slot-index))
+            (dpb hash (byte nbits 0) (type-hash-value obj)))))
+  obj)
+
 ;;;; object types to represent classes
 
 ;;; An UNDEFINED-CLASSOID is a cookie we make up to stick in forward
 ;;; referenced layouts. Users should never see them.
 (def!struct (undefined-classoid
              (:include classoid)
+             (:copier nil)
              (:constructor make-undefined-classoid (name))))
 
 ;;; BUILT-IN-CLASS is used to represent the standard classes that
@@ -258,14 +292,17 @@
 ;;; system operations (union, subtypep, etc.) should never encounter
 ;;; translated classes, only their translation.
 (def!struct (built-in-classoid (:include classoid)
-                               (:constructor make-built-in-classoid))
+                               (:copier nil)
+                               (:constructor !make-built-in-classoid))
   ;; the type we translate to on parsing. If NIL, then this class
   ;; stands on its own; or it can be set to :INITIALIZING for a period
   ;; during cold-load.
   (translation nil :type (or ctype (member nil :initializing))))
 
 (def!struct (condition-classoid (:include classoid)
-                                (:constructor make-condition-classoid))
+                                (:copier nil)
+                                (:constructor %make-condition-classoid
+                                    (hash-value name)))
   ;; list of CONDITION-SLOT structures for the direct slots of this
   ;; class
   (slots nil :type list)
@@ -289,12 +326,15 @@
   ;; Values for these slots must be computed in the dynamic
   ;; environment of MAKE-CONDITION.
   (hairy-slots nil :type list))
+(defun make-condition-classoid (&key name)
+  (%make-condition-classoid (interned-type-hash name) name))
 
 ;;;; classoid namespace
 
 ;;; We use an indirection to allow forward referencing of class
 ;;; definitions with load-time resolution.
 (def!struct (classoid-cell
+             (:copier nil)
              (:constructor make-classoid-cell (name &optional classoid))
              #-no-ansi-print-object
              (:print-object (lambda (s stream)
@@ -339,10 +379,13 @@
 ;;; side does not need to distinguish between STANDARD-CLASS and
 ;;; FUNCALLABLE-STANDARD-CLASS.
 (def!struct (standard-classoid (:include classoid)
-                               (:constructor make-standard-classoid)))
+                               (:copier nil)
+                               (:constructor make-standard-classoid))
+  old-layouts)
 ;;; a metaclass for classes which aren't standardlike but will never
 ;;; change either.
 (def!struct (static-classoid (:include classoid)
+                             (:copier nil)
                              (:constructor make-static-classoid)))
 
 (declaim (freeze-type built-in-classoid condition-classoid
@@ -354,8 +397,7 @@
 (define-info-type (:type :compiler-layout)
   :type-spec (or layout null)
   :default (lambda (name)
-             (let ((class (find-classoid name nil)))
-               (when class (classoid-layout class)))))
+             (awhen (find-classoid name nil) (classoid-layout it))))
 
 (eval-when (#-sb-xc :compile-toplevel :load-toplevel :execute)
 (defun ftype-from-fdefn (name)
@@ -382,6 +424,3 @@
 (define-info-type (:function :type)
   :type-spec (or ctype defstruct-description (member :generic-function))
   :default #'ftype-from-fdefn)
-
-;;; A random place for this :-(
-#+sb-xc-host (setq *info-environment* (make-info-hashtable))
