@@ -13,15 +13,12 @@
 
 (in-package "SB!IMPL")
 
-(defvar *core-pathname* nil
+(defglobal *core-pathname* nil
   "The absolute pathname of the running SBCL core.")
 
-(defvar *runtime-pathname* nil
+(defglobal *runtime-pathname* nil
   "The absolute pathname of the running SBCL runtime.")
 
-;;; something not EQ to anything we might legitimately READ
-(define-load-time-global *eof-object* (make-symbol "EOF-OBJECT"))
-
 (eval-when (:compile-toplevel :load-toplevel :execute)
   (defconstant max-hash sb!xc:most-positive-fixnum))
 
@@ -94,6 +91,10 @@
                           (* max-offset sb!vm:n-word-bytes))
                        scale)))
 
+(declaim (inline align-up))
+(defun align-up (value granularity &aux (mask (1- granularity)))
+  (logandc2 (+ value mask) mask))
+
 #!+(or x86 x86-64)
 (defun displacement-bounds (lowtag element-size data-offset)
   (let* ((adjustment (- (* data-offset sb!vm:n-word-bytes) lowtag))
@@ -182,12 +183,22 @@
            (recurse x (list x) 0)
            t))))
 
-;;; Is X a (possibly-improper) list of at least N elements?
-(declaim (ftype (function (t index)) list-of-length-at-least-p))
-(defun list-of-length-at-least-p (x n)
-  (or (zerop n) ; since anything can be considered an improper list of length 0
-      (and (consp x)
-           (list-of-length-at-least-p (cdr x) (1- n)))))
+;;; Is LIST a (possibly-improper) list of at least LENGTH elements?
+(declaim (ftype (sfunction (t index) boolean) list-of-length-at-least-p))
+(defun list-of-length-at-least-p (list length)
+  (named-let rec ((rest list) (n length))
+    (declare (type index n))
+    (or (zerop n) ; since anything can be considered an improper list of length 0
+        (and (consp rest) (rec (cdr rest) (1- n))))))
+
+(defun sequence-of-length-at-least-p (sequence length)
+  (etypecase sequence
+    (list
+     (list-of-length-at-least-p sequence length))
+    (vector
+     (>= (length sequence) length))
+    (sequence
+     (>= (length sequence) length))))
 
 ;;; Is X is a positive prime integer?
 (defun positive-primep (x)
@@ -343,6 +354,16 @@
                  (rplacd splice (cdr x))))
             (t (setq splice x)))))) ; Move splice along to include element.
 
+;;; Delete just one item
+(defun delq1 (item list)
+  (do ((prev nil x)
+       (x list (cdr x)))
+      ((null x) list)
+    (when (eq item (car x))
+      (if (null prev)
+          (return (cdr x))
+          (rplacd prev (cdr x)))
+      (return list))))
 
 ;;; like (POSITION .. :TEST #'EQ):
 ;;;   Return the position of the first element EQ to ITEM.
@@ -368,18 +389,6 @@
 
 ;;;; miscellaneous iteration extensions
 
-;;; like Scheme's named LET
-;;;
-;;; (CMU CL called this ITERATE, and commented it as "the ultimate
-;;; iteration macro...". I (WHN) found the old name insufficiently
-;;; specific to remind me what the macro means, so I renamed it.)
-(defmacro named-let (name binds &body body)
-  (dolist (x binds)
-    (unless (proper-list-of-length-p x 2)
-      (error "malformed NAMED-LET variable spec: ~S" x)))
-  `(labels ((,name ,(mapcar #'first binds) ,@body))
-     (,name ,@(mapcar #'second binds))))
-
 (defun filter-dolist-declarations (decls)
   (mapcar (lambda (decl)
             `(declare ,@(remove-if
@@ -579,7 +588,7 @@ NOTE: This interface is experimental and subject to change."
 ;;;   a fixnum with at least (* 2 <hash-bits>) of information in it.
 ;;; :VALUES <n>
 ;;;   the number of return values cached for each function call
-(defvar *cache-vector-symbols* nil)
+(define-load-time-global *cache-vector-symbols* nil)
 
 (defun drop-all-hash-caches ()
   (dolist (name *cache-vector-symbols*)
@@ -1085,6 +1094,8 @@ NOTE: This interface is experimental and subject to change."
 (defun defprinter-print-space (stream)
   (write-char #\space stream))
 
+(defvar *print-ir-nodes-pretty* nil)
+
 ;;; Define some kind of reasonable PRINT-OBJECT method for a
 ;;; STRUCTURE-OBJECT class.
 ;;;
@@ -1108,27 +1119,30 @@ NOTE: This interface is experimental and subject to change."
 ;;;
 ;;; The structure being printed is bound to STRUCTURE and the stream
 ;;; is bound to STREAM.
+;;;
+;;; If PRETTY-IR-PRINTER is supplied, the form is invoked when
+;;; *PRINT-IR-NODES-PRETTY* is true.
 (defmacro defprinter ((name
                        &key
                        (conc-name (concatenate 'simple-string
                                                (symbol-name name)
                                                "-"))
-                       identity)
+                       identity
+                       pretty-ir-printer)
                       &rest slot-descs)
   (let ((first? t)
         maybe-print-space
-        (reversed-prints nil)
-        (stream (sb!xc:gensym "STREAM")))
+        (reversed-prints nil))
     (flet ((sref (slot-name)
              `(,(symbolicate conc-name slot-name) structure)))
       (dolist (slot-desc slot-descs)
         (if first?
             (setf maybe-print-space nil
                   first? nil)
-            (setf maybe-print-space `(defprinter-print-space ,stream)))
+            (setf maybe-print-space `(defprinter-print-space stream)))
         (cond ((atom slot-desc)
                (push maybe-print-space reversed-prints)
-               (push `(defprinter-prin1 ',slot-desc ,(sref slot-desc) ,stream)
+               (push `(defprinter-prin1 ',slot-desc ,(sref slot-desc) stream)
                      reversed-prints))
               (t
                (let ((sname (first slot-desc))
@@ -1141,26 +1155,40 @@ NOTE: This interface is experimental and subject to change."
                                    ,maybe-print-space
                                    ,@(or (stuff)
                                          `((defprinter-prin1
-                                             ',sname ,sname ,stream)))))
+                                             ',sname ,sname stream)))))
                               reversed-prints))
                      (case (first option)
                        (:prin1
                         (stuff `(defprinter-prin1
-                                  ',sname ,(second option) ,stream)))
+                                  ',sname ,(second option) stream)))
                        (:princ
                         (stuff `(defprinter-princ
-                                  ',sname ,(second option) ,stream)))
+                                  ',sname ,(second option) stream)))
                        (:test (setq test (second option)))
                        (t
                         (error "bad option: ~S" (first option)))))))))))
-    `(defmethod print-object ((structure ,name) ,stream)
-       (pprint-logical-block (,stream nil)
-         (print-unreadable-object (structure
-                                   ,stream
-                                   :type t
-                                   :identity ,identity)
-           ,@(nreverse reversed-prints))))))
+    (let ((normal-printer `(pprint-logical-block (stream nil)
+                             (print-unreadable-object (structure
+                                                       stream
+                                                       :type t
+                                                       :identity ,identity)
+                               ,@(nreverse reversed-prints))) ))
+      `(defmethod print-object ((structure ,name) stream)
+         ,(cond (pretty-ir-printer
+                 `(if *print-ir-nodes-pretty*
+                      ,pretty-ir-printer
+                      ,normal-printer))
+                (t
+                 normal-printer))))))
 
+;;; When cross-compiling, there is nothing out of the ordinary
+;;; about compilling a DEFUN wrapped in PRESERVING-HOST-FUNCTION,
+;;; so just remove the decoration.
+#-sb-xc-host
+(eval-when (:compile-toplevel)
+  (sb!xc:defmacro sb-cold:preserving-host-function (form) form))
+
+(sb-cold:preserving-host-function
 (defun print-symbol-with-prefix (stream symbol &optional colon at)
   "For use with ~/: Write SYMBOL to STREAM as if it is not accessible from
   the current package."
@@ -1169,9 +1197,10 @@ NOTE: This interface is experimental and subject to change."
   ;; keywords are always printed with colons, so this guarantees that the
   ;; symbol will not be printed without a prefix.
   (let ((*package* *keyword-package*))
-    (write symbol :stream stream :escape t)))
+    (write symbol :stream stream :escape t))))
 
 (declaim (special sb!pretty:*pprint-quote-with-syntactic-sugar*))
+(sb-cold:preserving-host-function
 (defun print-type-specifier (stream type-specifier &optional colon at)
   (declare (ignore colon at))
   ;; Binding *PPRINT-QUOTE-WITH-SYNTACTIC-SUGAR* prevents certain
@@ -1187,10 +1216,11 @@ NOTE: This interface is experimental and subject to change."
   ;; specifiers.
   (let ((sb!pretty:*pprint-quote-with-syntactic-sugar* nil)
         (*package* *cl-package*))
-    (prin1 type-specifier stream)))
+    (prin1 type-specifier stream))))
 
+(sb-cold:preserving-host-function
 (defun print-type (stream type &optional colon at)
-  (print-type-specifier stream (type-specifier type) colon at))
+  (print-type-specifier stream (type-specifier type) colon at)))
 
 (declaim (ftype (sfunction (index &key (:comma-interval (and (integer 1) index))) index)
                 decimal-with-grouped-digits-width))
@@ -1379,23 +1409,25 @@ NOTE: This interface is experimental and subject to change."
 ;;; - SB-C::STACK-ALLOCATE-VECTOR (policy)         since 1.0.19.7            -> Final: anytime
 ;;; - SB-C::STACK-ALLOCATE-VALUE-CELLS (policy)    since 1.0.19.7            -> Final: anytime
 
+(sb-cold:preserving-host-function
 (defun print-deprecation-replacements (stream replacements &optional colonp atp)
   (declare (ignore colonp atp))
   ;; I don't think this is callable during cross-compilation, is it?
+  ;; Anyway, the format string tokenizer can not handle APPLY on its own.
   (apply #'format stream
-         "~#[~;~
+         (sb!format:tokens "~#[~;~
              Use ~/sb-ext:print-symbol-with-prefix/ instead.~;~
              Use ~/sb-ext:print-symbol-with-prefix/ or ~
              ~/sb-ext:print-symbol-with-prefix/ instead.~:;~
              Use~@{~#[~; or~] ~
              ~/sb-ext:print-symbol-with-prefix/~^,~} instead.~
-           ~]"
-         replacements))
+           ~]")
+         replacements)))
 
 (defun print-deprecation-message (namespace name software version
                                   &optional replacements stream)
   (format stream
-           "The ~(~A~) ~/sb!impl:print-symbol-with-prefix/ has been ~
+           "The ~(~A~) ~/sb!ext:print-symbol-with-prefix/ has been ~
             deprecated as of ~@[~A ~]version ~A.~
             ~@[~2%~/sb!impl::print-deprecation-replacements/~]"
           namespace name software version replacements))
@@ -1533,34 +1565,6 @@ NOTE: This interface is experimental and subject to change."
   (or (not (consp promise))
       (car promise)))
 
-;;; toplevel helper
-(defmacro with-rebound-io-syntax (&body body)
-  `(%with-rebound-io-syntax (lambda () ,@body)))
-
-(defun %with-rebound-io-syntax (function)
-  (declare (type function function))
-  (let ((*package* *package*)
-        (*print-array* *print-array*)
-        (*print-base* *print-base*)
-        (*print-case* *print-case*)
-        (*print-circle* *print-circle*)
-        (*print-escape* *print-escape*)
-        (*print-gensym* *print-gensym*)
-        (*print-length* *print-length*)
-        (*print-level* *print-level*)
-        (*print-lines* *print-lines*)
-        (*print-miser-width* *print-miser-width*)
-        (*print-pretty* *print-pretty*)
-        (*print-radix* *print-radix*)
-        (*print-readably* *print-readably*)
-        (*print-right-margin* *print-right-margin*)
-        (*read-base* *read-base*)
-        (*read-default-float-format* *read-default-float-format*)
-        (*read-eval* *read-eval*)
-        (*read-suppress* *read-suppress*)
-        (*readtable* *readtable*))
-    (funcall function)))
-
 ;;; Bind a few "potentially dangerous" printer control variables to
 ;;; safe values, respecting current values if possible.
 (defmacro with-sane-io-syntax (&body forms)
@@ -1573,6 +1577,7 @@ Does not affect the cases that are already controlled by *PRINT-LENGTH*")
 
 (defun call-with-sane-io-syntax (function)
   (declare (type function function))
+  #-sb-xc-host (declare (dynamic-extent function)) ; "unable"
   (macrolet ((true (sym)
                `(and (boundp ',sym) ,sym)))
     (let ((*print-readably* nil)
@@ -1729,3 +1734,5 @@ to :INTERPRET, an interpreter will be used.")
     (symbol (or (eq x t) (eq (symbol-package x) *keyword-package*)))
     (cons nil)
     (t t)))
+
+(defvar *top-level-form-p* nil)
