@@ -272,9 +272,13 @@ code to be loaded.
 ;;; While in theory we might want to include all of them,
 ;;; in practice it seems silly to allow "for x arithmetic-error in ..."
 (defun std-atom-type-specifier-p (symbol)
-  (and (eq (symbol-package symbol) *cl-package*)
-       ;; Check for primitiveness of the type, since some symbols in the CL
-       ;; package are private-use typedefs (e.g. ARRAY-RANK, CHAR-CODE).
+  ;; The check for symbols in CL is an optimization that skips calling INFO on
+  ;; most atoms. The real test is whether type = :PRIMITIVE, as there are CL
+  ;; symbols naming types which are not standard builtin type specifiers, e.g.
+  ;; ARRAY-RANK, CHAR-CODE. (One almost wonders if that is technically wrong).
+  ;; At any rate, in the cross-compiler, we must not rely on the host's idea
+  ;; of SYMBOL-PACKAGE as it does not accurately model the target per se.
+  (and #-sb-xc-host (eq (symbol-package symbol) *cl-package*)
        (or (eq (info :type :kind symbol) :primitive)
            ;; allow certain :instance types, but not all of them
            (member symbol '(hash-table package pathname random-state readtable)))
@@ -317,26 +321,44 @@ code to be loaded.
                            ,(loop-make-psetq (cddr frobs))))))))
 
 (defun loop-make-desetq (var-val-pairs)
-  (if (null var-val-pairs)
-      nil
-      (cons 'loop-really-desetq var-val-pairs)))
+  (if var-val-pairs (cons 'loop-desetq var-val-pairs)))
 
 (sb!ext:defglobal *loop-desetq-temporary*
         (make-symbol "LOOP-DESETQ-TEMP"))
 
-(sb!xc:defmacro loop-really-desetq (&environment env
-                                               &rest var-val-pairs)
+(sb!xc:defmacro loop-desetq (&environment env &rest var-val-pairs)
   (labels ((find-non-null (var)
              ;; See whether there's any non-null thing here. Recurse
              ;; if the list element is itself a list.
              (do ((tail var)) ((not (consp tail)) tail)
                (when (find-non-null (pop tail)) (return t))))
            (loop-desetq-internal (var val &optional temp)
+             ;; Check for well-formed use of MULTIPLE-VALUE-LIST
+             (when (and (typep val '(cons (eql multiple-value-list) (cons t null)))
+                        (proper-list-p var))
+               (return-from loop-desetq-internal
+                 (let ((temps (make-gensym-list (length var))))
+                   `((multiple-value-bind ,temps ,(cadr val)
+                       ,@(when (member nil var)
+                           `((declare (ignore ,@(mapcan (lambda (var val)
+                                                          (unless var (list val)))
+                                                        var temps)))))
+                       ,@(mapcan (lambda (var val)
+                                   (if var (list `(loop-desetq ,var ,val))))
+                                 var temps))))))
              ;; returns a list of actions to be performed
              (typecase var
                (null
                  (when (consp val)
                    ;; Don't lose possible side effects.
+                   ;; FIXME: this special case is just wrong. Either (DESETQ NIL THING)
+                   ;; evaluates THING, or it doesn't. If it does, then it mustn't touch
+                   ;; THING at all, because the user could have written
+                   ;;  (LOOP FOR NIL = (PROG1 AAA BBB)) where AAA and BBB are
+                   ;; symbol-macros the expansions of which are, let's say, (INCF C)
+                   ;; and (WHEN (PLUSP C) (RETURN)) respectively. The desired effect is
+                   ;; to perform the incf, ignore it, and then return.  But we turn that
+                   ;; whole form into NIL because of a PROG1. Wtf???
                    (if (eq (car val) 'prog1)
                        ;; These can come from PSETQ or DESETQ below.
                        ;; Throw away the value, keep the side effects.
@@ -539,9 +561,8 @@ code to be loaded.
 
 (defun loop-error (format-string &rest format-args)
   #-sb-xc-host(declare (optimize allow-non-returning-tail-call))
-  (error 'simple-program-error
-         :format-control "~?~%current LOOP context:~{ ~S~}."
-         :format-arguments (list format-string format-args (loop-context))))
+  (%program-error "~?~%current LOOP context:~{ ~S~}."
+                  format-string format-args (loop-context)))
 
 (defun loop-warn (format-string &rest format-args)
   (warn "~?~%current LOOP context:~{ ~S~}."
@@ -1328,30 +1349,21 @@ code to be loaded.
     (multiple-value-bind (vector-form constantp vector-value)
         (loop-constant-fold-if-possible val 'vector)
       (loop-make-var
-        vector-var vector-form
-        (if (and (consp vector-form) (eq (car vector-form) 'the))
-            (cadr vector-form)
-            'vector))
+       vector-var vector-form
+       (if (and (consp vector-form) (eq (car vector-form) 'the))
+           (cadr vector-form)
+           'vector))
       (loop-make-var index-var 0 'fixnum)
-      (let* ((length 0)
-             (length-form (cond ((not constantp)
-                                 (let ((v (gensym "LOOP-ACROSS-LIMIT-")))
-                                   (push `(setq ,v (length ,vector-var))
-                                         (prologue *loop*))
-                                   (loop-make-var v 0 'fixnum)))
-                                (t (setq length (length vector-value)))))
-             (first-test `(>= ,index-var ,length-form))
-             (other-test first-test)
+      (let* ((length-form (if constantp
+                              (length vector-value)
+                              (let ((v (gensym "LOOP-ACROSS-LIMIT-")))
+                                   (push `(let ((,v (length ,vector-var))))
+                                         (wrappers *loop*))
+                                   v)))
+             (test `(>= ,index-var ,length-form))
              (step `(,var (aref ,vector-var ,index-var)))
              (pstep `(,index-var (1+ ,index-var))))
-        (declare (fixnum length))
-        (when constantp
-          (setq first-test (= length 0))
-          (when (<= length 1)
-            (setq other-test t)))
-        `(,other-test ,step () ,pstep
-          ,@(and (neq first-test other-test)
-                 `(,first-test ,step () ,pstep)))))))
+        `(,test ,step () ,pstep)))))
 
 ;;;; list iteration
 
@@ -1410,7 +1422,7 @@ code to be loaded.
       (loop-constant-fold-if-possible val)
     (let ((listvar (gensym "LOOP-LIST-")))
       (loop-make-var var nil data-type)
-      (loop-make-var listvar list 'list)
+      (loop-make-var listvar list t)
       (let ((list-step (loop-list-step listvar)))
         (let* ((first-endtest `(endp ,listvar))
                (other-endtest first-endtest)
@@ -1579,7 +1591,8 @@ code to be loaded.
          (limit-given nil) ; T when prep phrase has specified end
          (limit-constantp nil)
          (limit-value nil))
-     #+ccl (progn start-constantp start-value) ; bogus "Unused" warnings
+     ;; Work around some "Unused" (rather, assigned-but-never-set) warnings
+     #+(or ccl clisp) (declare (ignorable start-constantp start-value))
      (flet ((assert-index-for-arithmetic (index)
               (unless (atom index)
                 (loop-error "Arithmetic index must be an atom."))))
