@@ -57,10 +57,11 @@
 variable: an unreadable object representing the error is printed instead.")
 
 ;; duplicate defglobal because this file is compiled before "reader"
-(defglobal *standard-readtable* nil)
+(define-load-time-global *standard-readtable* nil)
 
 (defun %with-standard-io-syntax (function)
   (declare (type function function))
+  (declare (dynamic-extent function))
   (let ((*package* (load-time-value (find-package "COMMON-LISP-USER") t))
         (*print-array* t)
         (*print-base* 10)
@@ -82,7 +83,8 @@ variable: an unreadable object representing the error is printed instead.")
         (*read-eval* t)
         (*read-suppress* nil)
         (*readtable* *standard-readtable*)
-        (*suppress-print-errors* nil))
+        (*suppress-print-errors* nil)
+        (*print-vector-length* nil))
     (funcall function)))
 
 ;;;; routines to print objects
@@ -246,11 +248,12 @@ variable: an unreadable object representing the error is printed instead.")
       o)))
 
 ;;; guts of PRINT-UNREADABLE-OBJECT
-(defun %print-unreadable-object (object stream type identity &optional body)
+(defun %print-unreadable-object (object stream flags &optional body)
   (declare (type (or null function) body))
   (if *print-readably*
       (print-not-readable-error object stream)
-      (flet ((print-description ()
+      (flet ((print-description (&aux (type (logbitp 0 (truly-the (mod 4) flags)))
+                                      (identity (logbitp 1 flags)))
                (when type
                  (write (type-of object) :stream stream :circle nil
                                          :level nil :length nil)
@@ -455,7 +458,7 @@ variable: an unreadable object representing the error is printed instead.")
 ;;; fixnum with bits set corresponding to attributes that the
 ;;; character has. At characters have at least one bit set, so we can
 ;;; search for any character with a positive test.
-(defvar *character-attributes*
+(define-load-time-global *character-attributes*
   (make-array 160 ; FIXME
               :element-type '(unsigned-byte 16)
               :initial-element 0))
@@ -490,7 +493,7 @@ variable: an unreadable object representing the error is printed instead.")
 ;;; lowest base in which that character is a digit.
 (declaim (type (simple-array (unsigned-byte 8) (128)) ; FIXME: range?
                *digit-bases*))
-(defvar *digit-bases*
+(define-load-time-global *digit-bases*
   (make-array 128 ; FIXME
               :element-type '(unsigned-byte 8)))
 
@@ -823,35 +826,49 @@ variable: an unreadable object representing the error is printed instead.")
 
 (defmethod print-object ((vector vector) stream)
   (let ((readably *print-readably*))
-    (cond ((stringp vector)
-           (cond ((and readably (not (typep vector '(vector character))))
-                  (output-unreadable-array-readably vector stream))
-                 ((or *print-escape* readably)
-                  (write-char #\" stream)
-                  (quote-string vector stream)
-                  (write-char #\" stream))
-                 (t
-                  (write-string vector stream))))
-          ((not (or *print-array* readably))
-           (output-terse-array vector stream))
-          ((bit-vector-p vector)
-           (write-string "#*" stream)
-           (dovector (bit vector)
-             ;; (Don't use OUTPUT-OBJECT here, since this code
-             ;; has to work for all possible *PRINT-BASE* values.)
-             (write-char (if (zerop bit) #\0 #\1) stream)))
-          ((or (not readably) (array-readably-printable-p vector))
-           (descend-into (stream)
-             (write-string "#(" stream)
-             (dotimes (i (length vector))
-               (unless (zerop i)
-                 (write-char #\space stream))
-               (punt-print-if-too-long i stream)
-               (output-object (aref vector i) stream))
-             (write-string ")" stream)))
+    (flet ((cut-length ()
+             (when (and (not readably)
+                        *print-vector-length*
+                        (> (length vector) *print-vector-length*))
+               (print-unreadable-object (vector stream :type t :identity t)
+                 (format stream "~A..."
+                         (make-array *print-vector-length*
+                                     :element-type (array-element-type vector)
+                                     :displaced-to vector)))
+               t)))
+      (cond ((stringp vector)
+             (cond ((and readably (not (typep vector '(vector character))))
+                    (output-unreadable-array-readably vector stream))
+                   ((and *print-escape*
+                         (cut-length)))
+                   ((or *print-escape* readably)
+                    (write-char #\" stream)
+                    (quote-string vector stream)
+                    (write-char #\" stream))
+                   (t
+                    (write-string vector stream))))
+            ((not (or *print-array* readably))
+             (output-terse-array vector stream))
+            ((bit-vector-p vector)
+             (cond ((cut-length))
+                   (t
+                    (write-string "#*" stream)
+                    (dovector (bit vector)
+                      ;; (Don't use OUTPUT-OBJECT here, since this code
+                      ;; has to work for all possible *PRINT-BASE* values.)
+                      (write-char (if (zerop bit) #\0 #\1) stream)))))
+            ((or (not readably) (array-readably-printable-p vector))
+             (descend-into (stream)
+               (write-string "#(" stream)
+               (dotimes (i (length vector))
+                 (unless (zerop i)
+                   (write-char #\space stream))
+                 (punt-print-if-too-long i stream)
+                 (output-object (aref vector i) stream))
+               (write-string ")" stream)))
 
-          (t
-           (output-unreadable-array-readably vector stream)))))
+            (t
+             (output-unreadable-array-readably vector stream))))))
 
 ;;; This function outputs a string quoting characters sufficiently
 ;;; so that someone can read it in again. Basically, put a slash in
@@ -995,7 +1012,7 @@ variable: an unreadable object representing the error is printed instead.")
 ;;;
 ;;; It doesn't need a lock, but if you work on SCRUB-POWER-CACHE or
 ;;; POWERS-FOR-BASE, see that you don't break the assumptions!
-(defglobal *power-cache* (make-array 37 :initial-element nil))
+(define-load-time-global *power-cache* (make-array 37 :initial-element nil))
 (declaim (type (simple-vector 37) *power-cache*))
 
 (defconstant +power-cache-integer-length-limit+ 2048)
@@ -1170,9 +1187,6 @@ variable: an unreadable object representing the error is printed instead.")
 
 (defun flonum-to-string (x &optional width fdigits scale fmin)
   (declare (type float x))
-  ;; FIXME: I think only FORMAT-DOLLARS calls FLONUM-TO-STRING with
-  ;; possibly-negative X.
-  (setf x (abs x))
   (multiple-value-bind (e string)
       (if fdigits
           (flonum-to-digits x (min (- (+ fdigits (or scale 0)))
@@ -1518,21 +1532,20 @@ variable: an unreadable object representing the error is printed instead.")
          (%output-integer-in-base exp 10 stream))))
 
 (defun output-float-infinity (x stream)
-  (declare (float x) (stream stream))
-  (cond (*read-eval*
-         (write-string "#." stream))
-        (*print-readably*
-         (return-from output-float-infinity
-           (print-not-readable-error x stream)))
-        (t
-         (write-string "#<" stream)))
-  (write-string "SB-EXT:" stream)
-  (write-string (symbol-name (float-format-name x)) stream)
-  (write-string (if (plusp x) "-POSITIVE-" "-NEGATIVE-")
-                stream)
-  (write-string "INFINITY" stream)
-  (unless *read-eval*
-    (write-string ">" stream)))
+  (declare (stream stream))
+  (let ((symbol (etypecase x
+                  (single-float (if (minusp x)
+                                    'single-float-negative-infinity
+                                    'single-float-positive-infinity))
+                  (double-float (if (minusp x)
+                                    'double-float-negative-infinity
+                                    'double-float-positive-infinity)))))
+    (cond (*read-eval*
+           (write-string "#." stream)
+           (output-symbol symbol (symbol-package symbol) stream))
+          (t
+           (print-unreadable-object (x stream)
+             (output-symbol symbol (symbol-package symbol) stream))))))
 
 (defun output-float-nan (x stream)
   (print-unreadable-object (x stream)
@@ -1601,11 +1614,17 @@ variable: an unreadable object representing the error is printed instead.")
 
 (defmethod print-object ((component code-component) stream)
   (print-unreadable-object (component stream :identity t)
-    (let ((dinfo (%code-debug-info component)))
-      (cond ((eq dinfo :bogus-lra)
+    (let (dinfo)
+      (cond ((code-obj-is-filler-p component)
+             (format stream "filler ~db" (%code-code-size component)))
+            ((eq (setq dinfo (%code-debug-info component)) :bogus-lra)
              (write-string "bogus code object" stream))
+            ((functionp dinfo)
+             (format stream "trampoline ~S" dinfo))
             (t
-             (format stream "code object [~D]" (code-n-entries component))
+             (format stream "code~@[ id=~x~] [~D]"
+                     (%code-serialno component)
+                     (code-n-entries component))
              (let ((fun-name (awhen (%code-entry-point component 0)
                                (%simple-fun-name it))))
                (when fun-name
@@ -1645,16 +1664,18 @@ variable: an unreadable object representing the error is printed instead.")
                 (values "#.(~S #X~16,'0X #X~16,'0X)"
                         '%make-simd-pack-ub64 #'%simd-pack-ub64s)))
            (multiple-value-call
-            #'format stream format maker (funcall extractor pack))))
+               #'format stream format maker (funcall extractor pack))))
+        (*print-readably*
+         (print-not-readable-error pack stream))
         (t
          (print-unreadable-object (pack stream)
            (flet ((all-ones-p (value start end &aux (mask (- (ash 1 end) (ash 1 start))))
-                      (= (logand value mask) mask))
+                    (= (logand value mask) mask))
                   (split-num (value start)
-                      (loop
-                         for i from 0 to 3
-                         and v = (ash value (- start)) then (ash v -8)
-                         collect (logand v #xFF))))
+                    (loop
+                       for i from 0 to 3
+                       and v = (ash value (- start)) then (ash v -8)
+                       collect (logand v #xFF))))
              (multiple-value-bind (low high)
                  (%simd-pack-ub64s pack)
                (etypecase pack
@@ -1694,41 +1715,42 @@ variable: an unreadable object representing the error is printed instead.")
 
 ;;;; catch-all for unknown things
 
+(declaim (inline lowtag-of))
+(defun lowtag-of (x) (logand (get-lisp-obj-address x) sb!vm:lowtag-mask))
+
 (defmethod print-object ((object t) stream)
-  (flet ((output-it (stream)
-          (when (eq object sb!pcl:+slot-unbound+)
-            (print-unreadable-object (object stream)
-              ;; If specifically the unbound marker with 0 data,
-              ;; as opposed to any other unbound marker.
-              (write-string "unbound" stream))
-            (return-from output-it))
-           (print-unreadable-object (object stream :identity t)
-             (let ((lowtag (lowtag-of object)))
-               (case lowtag
-                 (#.sb!vm:other-pointer-lowtag
-                  (let ((widetag (widetag-of object)))
-                    (case widetag
-                      (#.sb!vm:value-cell-widetag
-                       (write-string "value cell " stream)
-                       (output-object (value-cell-ref object) stream))
-                      (t
-                       (write-string "unknown pointer object, widetag=" stream)
-                       (output-integer widetag stream 16 t)))))
-                 ((#.sb!vm:fun-pointer-lowtag
-                   #.sb!vm:instance-pointer-lowtag
-                   #.sb!vm:list-pointer-lowtag)
-                  (write-string "unknown pointer object, lowtag=" stream)
-                  (output-integer lowtag stream 16 t))
-                (t
-                 (case (widetag-of object)
-                   (#.sb!vm:unbound-marker-widetag
-                    (write-string "unbound marker" stream))
-                   (t
-                    (write-string "unknown immediate object, lowtag=" stream)
-                    (output-integer lowtag stream 2 t)
-                    (write-string ", widetag=" stream)
-                    (output-integer (widetag-of object) stream 16 t)))))))))
-    (if *print-pretty*
-      ;; This block might not be necessary. Not sure, probably can't hurt.
-        (pprint-logical-block (stream nil) (output-it stream))
-        (output-it stream))))
+  (when (eq object sb!pcl:+slot-unbound+)
+    ;; If specifically the unbound marker with 0 data,
+    ;; as opposed to any other unbound marker.
+    (print-unreadable-object (object stream) (write-string "unbound" stream))
+    (return-from print-object))
+  (print-unreadable-object (object stream :identity t)
+    (let ((lowtag (lowtag-of object)))
+      (case lowtag
+        (#.sb!vm:other-pointer-lowtag
+         (let ((widetag (widetag-of object)))
+           (case widetag
+             (#.sb!vm:value-cell-widetag
+              (write-string "value cell " stream)
+              (output-object (value-cell-ref object) stream))
+             (#.sb!vm:filler-widetag
+              (write-string "pad " stream)
+              (write (1+ (get-header-data object)) :stream stream)
+              (write-string "w" stream)) ; words
+             (t
+              (write-string "unknown pointer object, widetag=" stream)
+              (output-integer widetag stream 16 t)))))
+        ((#.sb!vm:fun-pointer-lowtag
+          #.sb!vm:instance-pointer-lowtag
+          #.sb!vm:list-pointer-lowtag)
+         (write-string "unknown pointer object, lowtag=" stream)
+         (output-integer lowtag stream 16 t))
+        (t
+         (case (widetag-of object)
+           (#.sb!vm:unbound-marker-widetag
+            (write-string "unbound marker" stream))
+           (t
+            (write-string "unknown immediate object, lowtag=" stream)
+            (output-integer lowtag stream 2 t)
+            (write-string ", widetag=" stream)
+            (output-integer (widetag-of object) stream 16 t))))))))
