@@ -25,20 +25,20 @@
 
 (declaim (type list *save-hooks* *init-hooks* *exit-hooks*))
 
-(defvar *save-hooks* nil
+(define-load-time-global *save-hooks* nil
   "A list of function designators which are called in an unspecified
 order before creating a saved core image.
 
 Unused by SBCL itself: reserved for user and applications.")
 
-(defvar *init-hooks* nil
+(define-load-time-global *init-hooks* nil
   "A list of function designators which are called in an unspecified
 order when a saved core image starts up, after the system itself has
 been initialized.
 
 Unused by SBCL itself: reserved for user and applications.")
 
-(defvar *exit-hooks* nil
+(define-load-time-global *exit-hooks* nil
   "A list of function designators which are called in an unspecified
 order when SBCL process exits.
 
@@ -149,3 +149,21 @@ these hooks.")
                   (%shrink-vector string size)
                   string)))
          ,@body))))
+
+(defmacro with-locked-hash-table ((hash-table) &body body)
+  "Limits concurrent accesses to HASH-TABLE for the duration of BODY.
+If HASH-TABLE is synchronized, BODY will execute with exclusive
+ownership of the table. If HASH-TABLE is not synchronized, BODY will
+execute with other WITH-LOCKED-HASH-TABLE bodies excluded -- exclusion
+of hash-table accesses not surrounded by WITH-LOCKED-HASH-TABLE is
+unspecified."
+  ;; Needless to say, this also excludes some internal bits, but
+  ;; getting there is too much detail when "unspecified" says what
+  ;; is important -- unpredictable, but harmless.
+  `(sb!thread::with-recursive-lock ((hash-table-lock ,hash-table))
+     ,@body))
+
+(defmacro with-locked-system-table ((hash-table) &body body)
+  `(sb!thread::with-recursive-system-lock
+       ((hash-table-lock ,hash-table))
+     ,@body))
