@@ -81,12 +81,12 @@
   ;; dynamic-space-free-pointer increments only when a page is full.
   ;; Using boxed_region directly is finer-grained.
   #!+(and (not sb-thread) gencgc)
-  (ash (extern-alien "boxed_region" unsigned-long)
+  (ash (extern-alien "gc_alloc_region" unsigned-long)
        (- (1+ sb!vm:word-shift)))
   ;; threads imply gencgc. use the per-thread alloc region pointer
   #!+sb-thread
   (ash (sap-int (sb!vm::current-thread-offset-sap
-                 sb!vm::thread-alloc-region-slot))
+                 #.sb!vm::thread-alloc-region-slot))
        (- (1+ sb!vm:word-shift))))
 
 ;; Return some bits that are dependent on the next address that will be
@@ -189,6 +189,15 @@
        ;; Make sure we never return 0 (almost no chance of that anyway).
        (return answer)))))
 
+(declaim (inline !condition-hash))
+(defun !condition-hash (instance)
+  (let ((hash (sb!kernel::condition-hash instance)))
+    (if (not (eql hash 0))
+        hash
+        (let ((new (new-instance-hash-code)))
+          ;; At most one thread will compute a random hash.
+          (let ((old (cas (sb!kernel::condition-hash instance) 0 new)))
+            (if (eql old 0) new old))))))
 
 #!+(and compact-instance-header x86-64)
 (progn
@@ -202,6 +211,9 @@
           stored-hash))))
 
 (defun std-instance-hash (instance)
+  ;; Apparently we care that the object is of primitive type INSTANCE, but not
+  ;; whether it is STANDARD-INSTANCE. It had better be, or we're in trouble.
+  (declare (instance instance))
   #!+(and compact-instance-header x86-64)
   ;; The one logical slot (excluding layout) in the primitive object is index 0.
   ;; That holds a vector of the clos slots, and its header holds the hash.
@@ -212,23 +224,29 @@
     ;; (There are only 32 bits of actual randomness, if even that)
     (logxor (ash hash (- sb!vm:n-positive-fixnum-bits 32)) hash))
   #!-(and compact-instance-header x86-64)
-  (let ((hash (%instance-ref instance sb!pcl::std-instance-hash-slot-index)))
-    (if (not (eql hash 0))
-        hash
-        (let ((new (new-instance-hash-code)))
+  (locally
+   (declare (optimize (sb!c::type-check 0)))
+   (let ((hash (sb!pcl::standard-instance-hash-code instance)))
+     (if (not (eql hash 0))
+         hash
+         (let ((new (new-instance-hash-code)))
           ;; At most one thread will compute a random hash.
-          ;; %INSTANCE-CAS is a full call if there is no vop for it.
-          (let ((old (%instance-cas instance sb!pcl::std-instance-hash-slot-index
-                                    0 new)))
-            (if (eql old 0) new old))))))
+          (let ((old (cas (sb!pcl::standard-instance-hash-code instance) 0 new)))
+            (if (eql old 0) new old)))))))
 
 ;; These are also random numbers, but not lazily computed.
 (declaim (inline fsc-instance-hash))
 (defun fsc-instance-hash (fin)
-  #!+compact-instance-header
-  (sb!vm::get-header-data-high (%funcallable-instance-info fin 0))
-  #!-compact-instance-header
-  (%funcallable-instance-info fin sb!pcl::fsc-instance-hash-slot-index))
+  ;; As above, we care that the object is of primitive type FUNCTION, but not
+  ;; whether it is STANDARD-FUNCALLABLE-INSTANCE. Let's assume it is.
+  (declare (function fin))
+  (locally
+   (declare (optimize (sb!c::type-check 0)))
+   #!+compact-instance-header
+   (sb!vm::get-header-data-high
+    (sb!pcl::standard-funcallable-instance-clos-slots fin))
+   #!-compact-instance-header
+   (sb!pcl::standard-funcallable-instance-hash-code fin)))
 
 (defun sxhash (x)
   ;; profiling SXHASH is hard, but we might as well try to make it go
@@ -317,7 +335,7 @@
                            (sxhash      ; through DEFTRANSFORM
                             (classoid-name
                              (layout-classoid (%instance-layout x))))))
-                  (condition (sb!kernel::condition-hash x))
+                  (condition (!condition-hash x))
                   (t (std-instance-hash x))))
                (symbol (sxhash x))      ; through DEFTRANSFORM
                (array
@@ -339,7 +357,12 @@
                         (sxhash (char-code x)))) ; through DEFTRANSFORM
                ;; general, inefficient case of NUMBER
                (number (sxhash-number x))
-               (funcallable-instance (fsc-instance-hash x))
+               (funcallable-instance
+                (if (layout-for-std-class-p
+                     (%funcallable-instance-layout x))
+                    (fsc-instance-hash x)
+                    ;; funcallable structure, not funcallable-standard-object
+                    9550684))
                (t 42))))
     (sxhash-recurse x +max-hash-depthoid+)))
 
