@@ -364,6 +364,19 @@
                   (cons method more-methods)))
        ',name)))
 
+(defmacro get-dsd-index (type-name slot-name)
+  ;; Declare DD-SLOTS explicitly notinline, to avoid a warning, as its source
+  ;; appears later, in early-classoid. However CCL would warn that this
+  ;; declaration pertains to an unknown function, so only do this for us.
+  ;; You'd think we should also warn about DSD-INDEX, but the comment near
+  ;; that function says that it can't be inlined due to reasons.
+  ;; In make-host-2 everything is fine, because of DEF!STRUCT magic.
+  ;; And finally, we are prevented from writing "#+sbcl" here, for reasons.
+  #!+(host-feature sbcl) (declare (notinline dd-slots)) ; forward reference
+  (dsd-index (find slot-name
+                   (dd-slots (find-defstruct-description type-name))
+                   :key #'dsd-name)))
+
 (defmacro !define-type-class (name &key inherits
                                      (enumerable (unless inherits (must-supply-this))
                                                  enumerable-supplied-p)
@@ -398,13 +411,7 @@
     ;; which avoids a bootstrap problem: it's tricky to dump a type-class.
     #+sb-xc
     (let ((type-class-index
-           (position name *type-classes* :key #'type-class-name))
-          (slot-index
-           ;; KLUDGE: silence bogus warning that FIND "certainly" returns NIL
-           (locally (declare (notinline find))
-             (dsd-index (find 'class-info
-                              (dd-slots (find-defstruct-description 'ctype))
-                              :key #'dsd-name)))))
+           (position name *type-classes* :key #'type-class-name)))
       `(!cold-init-forms
         (let* ((backpatch-list (svref *type-classes* ,type-class-index))
                (type-class ,make-it))
@@ -417,7 +424,8 @@
           (dolist (instance backpatch-list)
             ;; Fixup the class first, in case fixing the hash needs the class.
             ;; (It doesn't currently, but just in case it does)
-            (setf (%instance-ref instance ,slot-index) type-class)
+            (setf (%instance-ref instance ,(get-dsd-index ctype class-info))
+                  type-class)
             (!improve-ctype-hash instance ',name)))))))
 
 ;;; Define the translation from a type-specifier to a type structure for
