@@ -486,7 +486,7 @@
 ;; FIXME: it would be idiomatic to use (DEFINE-INSTRUCTION-MACRO COMPOSITE ...)
 ;; instead of exporting another instruction-generating macro into SB!VM.
 ;; An invocation would resemble (INST COMPOSITE {ADD|SUB|whatever| ARGS ...)
-(defmacro composite-immediate-instruction (op r x y &key fixnumize neg-op invert-y invert-r single-op-op first-op first-no-source)
+(defmacro composite-immediate-instruction (op r x y &key fixnumize neg-op invert-y invert-r single-op-op first-op first-no-source temporary)
   ;; Successively applies 8-bit wide chunks of Y to X using OP storing the result in R.
   ;;
   ;; If FIXNUMIZE is true, Y is fixnumized before being used.
@@ -498,17 +498,22 @@
   ;; it is used for a single operation instead of OP.
   ;; If FIRST-OP is given, it is used in the first iteration instead of OP.
   ;; If FIRST-NO-SOURCE is given, there will be ne source register (X) in the first iteration.
+  ;; If TEMPORARY is given, it should be a non-descriptor register
+  ;; used for the accumulation of a temporary non-descriptor.  Only makes sense with INVERT-R
+  (when temporary
+    (aver invert-r))
   (let ((bytespec (gensym "bytespec"))
         (value (gensym "value"))
-        (transformed (gensym "transformed")))
+        (transformed (gensym "transformed"))
+        (acc (gensym "acc")))
     (labels ((instruction (source-reg op neg-op &optional no-source)
                `(,@(if neg-op
                         `((if (< ,y 0)
-                              (inst ,neg-op ,r ,@(when (not no-source)`(,source-reg))
+                              (inst ,neg-op ,acc ,@(when (not no-source)`(,source-reg))
                                     (mask-field ,bytespec ,value))
-                              (inst ,op ,r ,@(when (not no-source) `(,source-reg))
+                              (inst ,op ,acc ,@(when (not no-source) `(,source-reg))
                                     (mask-field ,bytespec ,value))))
-                        `((inst ,op ,r ,@(when (not no-source) `(,source-reg))
+                        `((inst ,op ,acc ,@(when (not no-source) `(,source-reg))
                                 (mask-field ,bytespec ,value))))
                   (setf (ldb ,bytespec ,value) 0)))
              (composite ()
@@ -517,9 +522,9 @@
                   (do ((,bytespec (byte 8 (logandc1 1 (lowest-set-bit-index ,value)))
                                   (byte 8 (logandc1 1 (lowest-set-bit-index ,value)))))
                       ((zerop ,value))
-                    ,@(instruction r op neg-op)
-                    ,@(when invert-r
-                            `((inst mvn ,r ,r)))))))
+                    ,@(instruction acc op neg-op))
+                  ,@(when invert-r
+                      `((inst mvn ,r ,acc))))))
       `(let* ((,transformed ,(if fixnumize
                                  `(fixnumize ,y)
                                  `,y))
@@ -528,7 +533,8 @@
                                  `((if (< ,transformed 0) (- ,transformed) ,transformed))
                                  (if invert-y
                                      `((lognot ,transformed))
-                                     `(,transformed))))))
+                                   `(,transformed)))))
+              (,acc (or ,temporary ,r)))
          ,@(if single-op-op
               `((if (encodable-immediate ,transformed)
                     (inst ,single-op-op ,r ,x ,transformed)
