@@ -59,6 +59,16 @@
   (:results (res :scs (any-reg descriptor-reg)))
   (:generator 1
     (load-symbol-value res *binding-stack-pointer*)))
+
+(define-vop (current-nsp)
+  (:results (res :scs (any-reg descriptor-reg)))
+  (:generator 1
+    (move res nsp-tn)))
+
+(define-vop (set-nsp)
+  (:args (nsp :scs (any-reg descriptor-reg)))
+  (:generator 1
+    (move nsp-tn nsp)))
 
 ;;;; Unwind block hackery:
 
@@ -75,9 +85,9 @@
     (composite-immediate-instruction add block cfp-tn
                                      (* (tn-offset tn) n-word-bytes))
     (load-symbol-value temp *current-unwind-protect-block*)
-    (storew temp block unwind-block-current-uwp-slot)
-    (storew cfp-tn block unwind-block-current-cont-slot)
-    (storew code-tn block unwind-block-current-code-slot)
+    (storew temp block unwind-block-uwp-slot)
+    (storew cfp-tn block unwind-block-cfp-slot)
+    (storew code-tn block unwind-block-code-slot)
     (inst compute-lra temp lip entry-label)
     (storew temp block catch-block-entry-pc-slot)))
 
@@ -95,9 +105,9 @@
     (composite-immediate-instruction
      add result cfp-tn (* (tn-offset tn) n-word-bytes))
     (load-symbol-value temp *current-unwind-protect-block*)
-    (storew temp result catch-block-current-uwp-slot)
-    (storew cfp-tn result catch-block-current-cont-slot)
-    (storew code-tn result catch-block-current-code-slot)
+    (storew temp result catch-block-uwp-slot)
+    (storew cfp-tn result catch-block-cfp-slot)
+    (storew code-tn result catch-block-code-slot)
     (inst compute-lra temp lip entry-label)
     (storew temp result catch-block-entry-pc-slot)
 
@@ -108,15 +118,12 @@
 
     (move block result)))
 
-;;; Just set the current unwind-protect to TN's address.  This
+;;; Just set the current unwind-protect to UWP.  This
 ;;; instantiates an unwind block as an unwind-protect.
 (define-vop (set-unwind-protect)
-  (:args (tn))
-  (:temporary (:scs (descriptor-reg)) new-uwp)
+  (:args (uwp :scs (any-reg)))
   (:generator 7
-    (composite-immediate-instruction
-     add new-uwp cfp-tn (* (tn-offset tn) n-word-bytes))
-    (store-symbol-value new-uwp *current-unwind-protect-block*)))
+    (store-symbol-value uwp *current-unwind-protect-block*)))
 
 (define-vop (unlink-catch-block)
   (:temporary (:scs (any-reg)) block)
@@ -133,7 +140,7 @@
   (:translate %unwind-protect-breakup)
   (:generator 17
     (load-symbol-value block *current-unwind-protect-block*)
-    (loadw block block unwind-block-current-uwp-slot)
+    (loadw block block unwind-block-uwp-slot)
     (store-symbol-value block *current-unwind-protect-block*)))
 
 ;;;; NLX entry VOPs:
@@ -252,10 +259,10 @@
       ;; Set up magic catch / UWP block.
 
       (loadw temp uwp sap-pointer-slot other-pointer-lowtag)
-      (storew temp block unwind-block-current-uwp-slot)
+      (storew temp block unwind-block-uwp-slot)
       (loadw temp ofp sap-pointer-slot other-pointer-lowtag)
-      (storew temp block unwind-block-current-cont-slot)
-      ;; Don't need to save code at unwind-block-current-code-slot since
+      (storew temp block unwind-block-cfp-slot)
+      ;; Don't need to save code at unwind-block-code-slot since
       ;; it's not going to be used and will be overwritten after the
       ;; function call
 
@@ -263,7 +270,7 @@
       (storew temp block catch-block-entry-pc-slot)
 
       ;; Run any required UWPs.
-      (assemble (*elsewhere* vop)
+      (assemble (:elsewhere vop)
         (emit-label uwp-label)
         (inst word (make-fixup 'unwind :assembly-routine)))
       (inst load-from-label pc-tn lr-tn uwp-label)
