@@ -459,7 +459,6 @@
   (def fast-ash-left/signed=>signed signed-reg signed-num signed-reg 3)
   (def fast-ash-left/unsigned=>unsigned unsigned-reg unsigned-num unsigned-reg 3))
 
-#!+ash-right-vops
 (define-vop (fast-%ash/right/unsigned)
   (:translate %ash/right)
   (:policy :fast-safe)
@@ -471,7 +470,6 @@
   (:generator 4
      (inst lsr result number amount)))
 
-#!+ash-right-vops
 (define-vop (fast-%ash/right/signed)
   (:translate %ash/right)
   (:policy :fast-safe)
@@ -483,7 +481,6 @@
   (:generator 4
     (inst asr result number amount)))
 
-#!+ash-right-vops
 (define-vop (fast-%ash/right/fixnum)
   (:translate %ash/right)
   (:policy :fast-safe)
@@ -939,9 +936,8 @@
   (:result-types signed-num)
   (:info width)
   (:generator 3
-    (cond ((zerop width)
-           (inst mov r 0))
-          ((= width 64)
+    (aver (/= width 0))
+    (cond ((= width 64)
            (move r x))
           (t
            (inst sbfm r x 0 (1- width))))))
@@ -955,11 +951,9 @@
   (:result-types signed-num)
   (:info width)
   (:generator 4
-    (cond ((zerop width)
-           (inst mov r 0))
-          (t
-           (loadw r x bignum-digits-offset other-pointer-lowtag)
-           (inst sbfm r r 0 (1- width))))))
+    (aver (/= width 0))
+    (loadw r x bignum-digits-offset other-pointer-lowtag)
+    (inst sbfm r r 0 (1- width))))
 
 (define-vop (mask-signed-field-fixnum)
   (:translate sb!c::mask-signed-field)
@@ -1103,7 +1097,6 @@
     (inst umulh hi x y)
     (inst mul lo x y)))
 
-#!+multiply-high-vops
 (define-vop (mulhi)
   (:translate %multiply-high)
   (:policy :fast-safe)
@@ -1115,7 +1108,6 @@
   (:generator 20
     (inst umulh hi x y)))
 
-#!+multiply-high-vops
 (define-vop (mulhi/fx)
   (:translate %multiply-high)
   (:policy :fast-safe)
