@@ -78,7 +78,7 @@
 (define-move-fun (load-system-area-pointer 1) (vop x y)
   ((immediate) (sap-reg))
   (let ((immediate-label (gen-label)))
-    (assemble (*elsewhere*)
+    (assemble (:elsewhere)
       (emit-label immediate-label)
       (inst dword (sap-int (tn-value x))))
     (inst ldr y (@ immediate-label))))
@@ -187,25 +187,38 @@
              (load-arg (x load-tn)
                (sc-case x
                  (null
-                  null-tn)
+                  (lambda () null-tn))
                  ((constant immediate control-stack)
-                  (let ((tn (if (and used-load-tn
-                                     (location= used-load-tn
-                                                load-tn))
-                                tmp-tn
-                                load-tn)))
+                  (let ((load-tn (if (and used-load-tn
+                                          (location= used-load-tn load-tn))
+                                     tmp-tn
+                                     load-tn)))
+                    (setf used-load-tn load-tn)
                     (sc-case x
                       (constant
-                       (load-constant vop1 x tn))
-                      (immediate
-                       (load-immediate vop1 x tn))
+                       (when (eq load-tn tmp-tn)
+                         ;; TMP-TN is not a descriptor
+                         (return-from load-arg))
+                       (lambda ()
+                         (load-constant vop1 x load-tn)
+                         load-tn))
                       (control-stack
-                       (load-stack vop1 x tn)))
-                    (setf used-load-tn tn)
-                    tn))
+                       (when (eq load-tn tmp-tn)
+                         (return-from load-arg))
+                       (lambda ()
+                         (load-stack vop1 x load-tn)
+                         load-tn))
+                      (immediate
+                       (cond ((eql (tn-value x) 0)
+                              (setf used-load-tn nil)
+                              (lambda () zr-tn))
+                             (t
+                              (lambda ()
+                                (load-immediate vop1 x load-tn)
+                                load-tn)))))))
                  (t
                   (setf used-load-tn x)
-                  x)))
+                  (lambda () x))))
              (do-moves (source1 source2 dest1 dest2 &optional (fp cfp-tn)
                                                               fp-load-tn)
                (cond ((and (stack-p dest1)
@@ -216,19 +229,29 @@
                                (and (not (location= dest1 source2))
                                     (not (location= dest2 source1))))
                            (suitable-offsets-p dest1 dest2))
-                      (if (and (stack-p source1)
-                               (stack-p source2)
-                               (do-moves source1 source2
-                                 (load-tn vop1)
-                                 (if (location= (load-tn vop1) (load-tn vop2))
-                                     tmp-tn
-                                     (load-tn vop2))))
-                          (setf source1 (load-tn vop1)
-                                source2 (if (location= (load-tn vop1) (load-tn vop2))
-                                            tmp-tn
-                                            (load-tn vop2)))
-                          (setf source1 (load-arg source1 (load-tn vop1))
-                                source2 (load-arg source2 (load-tn vop2))))
+                      ;; Load the source registers
+                      (let (new-source1 new-source2)
+                        (if (and (stack-p source1)
+                                 (stack-p source2)
+                                 ;; Can load using LDP
+                                 (do-moves source1 source2
+                                   (setf new-source1 (load-tn vop1))
+                                   (setf new-source2
+                                         (cond ((not (location= (load-tn vop1) (load-tn vop2)))
+                                                (load-tn vop2))
+                                               ((sc-is (load-tn vop2) descriptor-reg)
+                                                (return-from do-moves))
+                                               (t
+                                                tmp-tn)))))
+                            (setf source1 new-source1
+                                  source2 new-source2)
+                            ;; Load one by one
+                            (let ((load1 (load-arg source1 (load-tn vop1)))
+                                  (load2 (load-arg source2 (load-tn vop2))))
+                              (unless (and load1 load2)
+                                (return-from do-moves))
+                              (setf source1 (funcall load1)
+                                    source2 (funcall load2)))))
                       (when (> (tn-offset dest1)
                                (tn-offset dest2))
                         (rotatef dest1 dest2)
@@ -249,9 +272,6 @@
                                (tn-offset source2))
                         (rotatef dest1 dest2)
                         (rotatef source1 source2))
-                      (when fp-load-tn
-                        (load-stack-tn fp-load-tn fp)
-                        (setf fp fp-load-tn))
                       (inst ldp dest1 dest2
                             (@ fp (* (tn-offset source1) n-word-bytes)))
                       t))))
@@ -269,13 +289,13 @@
                (fp2 (tn-ref-tn (tn-ref-across (sb!c::vop-args vop2))))
                (dest1 (dest vop1))
                (dest2 (dest vop2)))
-           (do-moves (source vop1) (source vop2) (dest vop1) (dest vop2)
-             (if (and (stack-p dest1)
-                      (stack-p dest2)
-                      (eq fp1 fp2))
-                 fp1
-                 cfp-tn)
-             (tn-ref-load-tn (tn-ref-across (sb!c::vop-args vop1))))))))))
+           (when (eq fp1 fp2)
+             (do-moves (source vop1) (source vop2) (dest vop1) (dest vop2)
+               (if (and (stack-p dest1)
+                        (stack-p dest2))
+                   fp1
+                   cfp-tn)
+               (tn-ref-load-tn (tn-ref-across (sb!c::vop-args vop1)))))))))))
 
 
 ;;;; ILLEGAL-MOVE
@@ -379,6 +399,25 @@
 (define-move-vop move-from-signed :move
   (signed-reg) (descriptor-reg))
 
+(define-vop (move-from-fixnum+1)
+  (:args (x :scs (signed-reg unsigned-reg)))
+  (:results (y :scs (any-reg descriptor-reg)))
+  (:vop-var vop)
+  (:generator 4
+    (inst adds y x x)
+    (inst b :vc DONE)
+    (load-constant vop (emit-constant (1+ sb!xc:most-positive-fixnum))
+                   y)
+    DONE))
+
+(define-vop (move-from-fixnum-1 move-from-fixnum+1)
+  (:generator 4
+    (inst adds y x x)
+    (inst b :vc DONE)
+    (load-constant vop (emit-constant (1- sb!xc:most-negative-fixnum))
+                   y)
+    DONE))
+
 ;;; Check for fixnum, and possibly allocate one or two word bignum
 ;;; result.  Use a worst-case cost to make sure people know they may
 ;;; be number consing.
