@@ -12,19 +12,15 @@
 
 (in-package "SB!C")
 
-(defvar *args* ()
-  "This variable is bound to the format arguments when an error is signalled
-by BARF or BURP.")
-
+;;; FIXME: unsafe if we remove world-lock from around compilation
 (defvar *ignored-errors* (make-hash-table :test 'equal))
 
-;;; A definite inconsistency has been detected. Signal an error with
-;;; *args* bound to the list of the format args.
+;;; A definite inconsistency has been detected. Signal an error.
 (declaim (ftype (function (string &rest t) (values)) barf))
-(defun barf (string &rest *args*)
+(defun barf (string &rest args)
   (unless (gethash string *ignored-errors*)
     (restart-case
-        (apply #'error string *args*)
+        (apply #'error string args)
       (continue ()
         :report "Ignore this error.")
       (ignore-all ()
@@ -40,11 +36,11 @@ One of :WARN, :ERROR or :NONE.")
 ;;; Called when something funny but possibly correct is noticed.
 ;;; Otherwise similar to BARF.
 (declaim (ftype (function (string &rest t) (values)) burp))
-(defun burp (string &rest *args*)
+(defun burp (string &rest args)
   (declare (notinline warn)) ; See COMPILER-WARN for rationale
   (ecase *burp-action*
-    (:warn (apply #'warn string *args*))
-    (:error (apply #'cerror "press on anyway." string *args*))
+    (:warn (apply #'warn string args))
+    (:error (apply #'cerror "press on anyway." string args))
     (:none))
   (values))
 
@@ -573,10 +569,11 @@ One of :WARN, :ERROR or :NONE.")
         (barf "The WRITE-P in ~S isn't ~S." vop write-p))
       (unless (find-in #'tn-ref-next-ref ref vop-refs)
         (barf "~S not found in REFS for ~S" ref vop))
-      (unless (find-in #'tn-ref-next ref
-                       (if (tn-ref-write-p ref)
-                           (tn-writes (tn-ref-tn ref))
-                           (tn-reads (tn-ref-tn ref))))
+      (unless (or (eq (tn-kind (tn-ref-tn ref)) :unused)
+                  (find-in #'tn-ref-next ref
+                           (if (tn-ref-write-p ref)
+                               (tn-writes (tn-ref-tn ref))
+                               (tn-reads (tn-ref-tn ref)))))
         (barf "~S not found in reads/writes for its TN" ref))
 
       (let ((target (tn-ref-target ref)))
@@ -855,12 +852,14 @@ One of :WARN, :ERROR or :NONE.")
            (do ((scs scs (cdr scs))
                 (op ops (tn-ref-across op)))
                ((null scs))
-             (let ((load-tn (tn-ref-load-tn op)))
-               (unless (eq (svref (car scs)
-                                  (sc-number
-                                   (tn-sc
-                                    (or load-tn (tn-ref-tn op)))))
-                           t)
+             (let ((tn (tn-ref-tn op))
+                   (load-tn (tn-ref-load-tn op)))
+               (unless (or (eq (tn-kind tn) :unused)
+                           (eq (svref (car scs)
+                                      (sc-number
+                                       (tn-sc
+                                        (or load-tn tn))))
+                               t))
                  (barf "operand restriction not satisfied: ~S" op))))))
     (do-ir2-blocks (block component)
       (do ((vop (ir2-block-last-vop block) (vop-prev vop)))
@@ -906,7 +905,13 @@ One of :WARN, :ERROR or :NONE.")
     (global-var
      (format stream "~S {~A}" (leaf-debug-name leaf) (global-var-kind leaf)))
     (functional
-     (format stream "~S ~S" (type-of leaf) (functional-debug-name leaf)))))
+     (format stream "~S ~S ~S" (type-of leaf) (functional-debug-name leaf)
+             (mapcar #'leaf-debug-name
+                     (typecase leaf
+                       (clambda
+                        (lambda-vars leaf))
+                       (optional-dispatch
+                        (optional-dispatch-arglist leaf))))))))
 
 ;;; Attempt to find a block given some thing that has to do with it.
 (declaim (ftype (sfunction (t) cblock) block-or-lose))
@@ -951,6 +956,8 @@ One of :WARN, :ERROR or :NONE.")
       (format t " <deleted>"))
 
     (pprint-newline :mandatory)
+    (when (block-start-cleanup block)
+      (format t "cleanup ~s~%" (cleanup-kind (block-start-cleanup block))))
     (awhen (block-info block)
       (format t "start stack: ")
       (print-lvar-stack (ir2-block-start-stack it))
@@ -974,7 +981,8 @@ One of :WARN, :ERROR or :NONE.")
              (dolist (arg (basic-combination-args node))
                (if arg
                    (print-lvar arg)
-                   (format t "<none> ")))))
+                   (format t "<none> "))))
+           (format t "{derived ~a}" (type-specifier (basic-combination-derived-type node))))
           (cset
            (write-string "set ")
            (print-leaf (set-var node))
@@ -1030,7 +1038,9 @@ One of :WARN, :ERROR or :NONE.")
       (pprint-newline :mandatory))
     (let ((succ (block-succ block)))
       (format t "successors~{ c~D~}~%"
-              (mapcar (lambda (x) (cont-num (block-start x))) succ))))
+              (mapcar (lambda (x) (cont-num (block-start x))) succ)))
+    (when (block-end-cleanup block)
+      (format t "cleanup ~s~%" (cleanup-kind (block-end-cleanup block)))))
   (values))
 
 ;;; Print the guts of a TN. (logic shared between PRINT-OBJECT (TN T)
@@ -1044,7 +1054,8 @@ One of :WARN, :ERROR or :NONE.")
           (t
            (format stream "t~D" (tn-id tn))))
     (when (and (tn-sc tn) (tn-offset tn))
-      (format stream "[~A]" (location-print-name tn)))))
+      (format stream "[~A]" (location-print-name tn)))
+    (format stream " ~s" (tn-kind tn))))
 
 ;;; Print the TN-REFs representing some operands to a VOP, linked by
 ;;; TN-REF-ACROSS.
@@ -1162,7 +1173,7 @@ One of :WARN, :ERROR or :NONE.")
         (format t "~&~A...~%" condition))))
   (values))
 
-(defvar *list-conflicts-table* (make-hash-table :test 'eq))
+(defvar *list-conflicts-table*)
 
 ;;; Add all ALWAYS-LIVE TNs in BLOCK to the conflicts. TN is ignored
 ;;; when it appears in the global conflicts.
