@@ -49,7 +49,8 @@
         (,lambda-expr ,whole-var *lexenv*)
         (values))
       #-sb-xc-host
-      (install-guard-function ',name '(:special ,name) ,doc)
+      (progn (install-guard-function ',name '(:special ,name))
+             (setf (documentation (symbol-function ',name) t) ',doc))
            ;; FIXME: Evidently "there can only be one!" -- we overwrite any
            ;; other :IR1-CONVERT value. This deserves a warning, I think.
       (setf (info :function :ir1-convert ',name) #',fn-name)
@@ -118,7 +119,7 @@
                    (make-lambda-list llks nil req opt rest keys aux)
                    lambda-list)) ; otherwise use the original list
        (args (make-symbol "ARGS")))
-    `(setf (info :function :source-transform ',fun-name)
+    `(%define-source-transform ',fun-name
            (named-lambda (:source-transform ,fun-name)
                (,lambda-whole ,lambda-env &aux (,args (cdr ,lambda-whole)))
              ,@(if (not env) `((declare (ignore ,lambda-env))))
@@ -133,6 +134,10 @@
                        ,@inner-decls
                        (block ,(fun-name-block-name fun-name) ,@forms))
                    (values call pass)))))))
+(defun %define-source-transform (fun-name lambda)
+  (when (info :function :source-transform fun-name)
+    (warn "Redefining source-transform for ~S" fun-name))
+  (setf (info :function :source-transform fun-name) lambda))
 
 ;;;; lambda-list parsing utilities
 ;;;;
@@ -268,8 +273,8 @@
 ;;;   :IMPORTANT
 ;;;           - If the transform fails and :IMPORTANT is
 ;;;               NIL,       then never print an efficiency note.
-;;;               :SLIGHTLY, then print a note if SPEED>=INHIBIT-WARNINGS.
-;;;               T,         then print a note if SPEED>INHIBIT-WARNINGS.
+;;;               :SLIGHTLY, then print a note if SPEED>INHIBIT-WARNINGS.
+;;;               T,         then print a note if SPEED>=INHIBIT-WARNINGS.
 ;;;             :SLIGHTLY is the default.
 (defmacro deftransform (name (lambda-list &optional (arg-types '*)
                                           (result-type '*)
@@ -347,134 +352,6 @@
                                  ,important
                                  policy))))))
 
-;;;; DEFKNOWN and DEFOPTIMIZER
-
-;;; This macro should be the way that all implementation independent
-;;; information about functions is made known to the compiler.
-;;;
-;;; FIXME: The comment above suggests that perhaps some of my added
-;;; FTYPE declarations are in poor taste. Should I change my
-;;; declarations, or change the comment, or what?
-;;;
-;;; FIXME: DEFKNOWN is needed only at build-the-system time. Figure
-;;; out some way to keep it from appearing in the target system.
-;;;
-;;; Declare the function NAME to be a known function. We construct a
-;;; type specifier for the function by wrapping (FUNCTION ...) around
-;;; the ARG-TYPES and RESULT-TYPE. ATTRIBUTES is an unevaluated list
-;;; of boolean attributes of the function. See their description in
-;;; (!DEF-BOOLEAN-ATTRIBUTE IR1). NAME may also be a list of names, in
-;;; which case the same information is given to all the names. The
-;;; keywords specify the initial values for various optimizers that
-;;; the function might have.
-(defmacro defknown (name arg-types result-type &optional (attributes '(any))
-                    &body keys)
-  #-sb-xc-host
-  (when (member 'unsafe attributes)
-    (style-warn "Ignoring legacy attribute UNSAFE. Replaced by its inverse: DX-SAFE.")
-    (setf attributes (remove 'unsafe attributes)))
-  (when (and (intersection attributes '(any call unwind))
-             (intersection attributes '(movable)))
-    (error "function cannot have both good and bad attributes: ~S" attributes))
-
-  (when (member 'any attributes)
-    (setq attributes (union '(unwind) attributes)))
-  (when (member 'flushable attributes)
-    (pushnew 'unsafely-flushable attributes))
-  (multiple-value-bind (callable-map arg-types)
-      (make-callable-map arg-types attributes)
-    `(%defknown ',(if (and (consp name)
-                           (not (legal-fun-name-p name)))
-                      name
-                      (list name))
-                '(sfunction ,arg-types ,result-type)
-                (ir1-attributes ,@attributes)
-                (source-location)
-                :callable-map ,callable-map
-                ,@keys)))
-
-(defun make-callable-map (arg-types attributes)
-  (if (member 'call attributes)
-      (multiple-value-bind (llks required optional rest keys)
-          (parse-lambda-list
-           arg-types
-           :context :function-type
-           :accept (lambda-list-keyword-mask
-                    '(&optional &rest &key &allow-other-keys))
-           :silent t)
-        (let (vars
-              call-vars
-              arg-count-specified)
-          (labels ((callable-p (x)
-                     (member x '(callable function)))
-                   (process-var (x &optional (name (gensym)))
-                     (if (callable-p (if (consp x)
-                                         (car x)
-                                         x))
-                         (push (list* name
-                                      (cond ((consp x)
-                                             (setf arg-count-specified t)
-                                             (cdr x))))
-                               call-vars)
-                         (push name vars))
-                     name)
-                   (process-type (type)
-                     (if (and (consp type)
-                              (callable-p (car type)))
-                         (car type)
-                         type))
-                   (callable-rest-p (x)
-                     (and (consp x)
-                          (callable-p (car x))
-                          (eql (cadr x) '&rest))))
-            (let* (rest-var
-                   (lambda-list
-                     (cond ((find-if #'callable-rest-p required)
-                            (setf rest-var (gensym))
-                            `(,@(loop for var in required
-                                      collect (process-var var)
-                                      until (callable-rest-p var))
-                              &rest ,rest-var))
-                           (t
-                            `(,@(mapcar #'process-var required)
-                              ,@(and optional
-                                     `(&optional ,@(mapcar #'process-var optional)))
-                              ,@(and rest
-                                     `(&rest ,@(mapcar #'process-var rest)))
-                              ,@(and (ll-kwds-keyp llks)
-                                     `(&key ,@(loop for (key type) in keys
-                                                    for var = (gensym)
-                                                    do (process-var type var)
-                                                    collect `((,key ,var))))))))))
-
-              (assert call-vars)
-              (values
-               `(lambda (function ,@lambda-list)
-                  (declare (ignore ,@vars))
-                  ,@(loop for (x arg-count . options) in call-vars
-                          collect `(funcall function
-                                            ,x
-                                            ,@(and arg-count
-                                                   `(:arg-count
-                                                     ,(if (eq arg-count '&rest)
-                                                          `(length ,rest-var)
-                                                          arg-count)))
-                                            ,@(loop for (keyword value) on options by #'cddr
-                                                    collect keyword
-                                                    collect `',value))))
-               `(,@(mapcar #'process-type required)
-                 ,@(and optional
-                        `(&optional ,@(mapcar #'process-type optional)))
-                 ,@(and (ll-kwds-restp llks)
-                        `(&rest ,@rest))
-                 ,@(and (ll-kwds-keyp llks)
-                        `(&key
-                          ,@(loop for (key type) in keys
-                                  collect `(,key ,(process-type type)))))
-                 ,@(and (ll-kwds-allowp llks)
-                        '(&allow-other-keys))))))))
-      (values nil arg-types)))
-
 ;;; Create a function which parses combination args according to WHAT
 ;;; and LAMBDA-LIST, where WHAT is either a function name or a list
 ;;; (FUN-NAME KIND) and does some KIND of optimization.
@@ -756,8 +633,6 @@
 ;;; experimentation, not for ordinary use, so it should probably
 ;;; become conditional on SB-SHOW.
 
-(eval-when (#-sb-xc :compile-toplevel :load-toplevel :execute)
-
 (defstruct (event-info (:copier nil))
   ;; The name of this event.
   (name (missing-arg) :type symbol)
@@ -774,7 +649,7 @@
   (action nil :type (or function null)))
 
 ;;; A hashtable from event names to event-info structures.
-(defvar *event-info* (make-hash-table :test 'eq))
+(define-load-time-global *event-info* (make-hash-table :test 'eq))
 
 ;;; Return the event info for Name or die trying.
 (declaim (ftype (function (t) event-info) event-info-or-lose))
@@ -784,8 +659,6 @@
       (error "~S is not the name of an event." name))
     res))
 
-) ; EVAL-WHEN
-
 ;;; Return the number of times that EVENT has happened.
 (declaim (ftype (function (symbol) fixnum) event-count))
 (defun event-count (name)
@@ -826,7 +699,7 @@
 (defmacro defevent (name description &optional (level 0))
   (let ((var-name (symbolicate "*" name "-EVENT-INFO*")))
     `(eval-when (:compile-toplevel :load-toplevel :execute)
-       (defvar ,var-name
+       (define-load-time-global ,var-name
          (make-event-info :name ',name
                           :description ',description
                           :var ',var-name
@@ -884,6 +757,7 @@
                 (key #'identity)
                 (test #'eql))
   (declare (type function next key test))
+  ;; #-sb-xc-host (declare (dynamic-extent next key test)) ; emits "unable" note
   (do ((current list (funcall next current)))
       ((null current) nil)
     (when (funcall test (funcall key current) element)
@@ -899,6 +773,7 @@
                     (key #'identity)
                     (test #'eql))
   (declare (type function next key test))
+  ;; #-sb-xc-host (declare (dynamic-extent next key test)) ; emits "unable" note
   (do ((current list (funcall next current))
        (i 0 (1+ i)))
       ((null current) nil)
