@@ -115,8 +115,7 @@
     (emit-label start-lab)
     ;; Allocate function header.
     (inst simple-fun-header-word)
-    (dotimes (i (1- simple-fun-code-offset))
-      (inst word 0))
+    (inst .skip (* (1- simple-fun-code-offset) n-word-bytes))
     (let ((entry-point (gen-label)))
       (emit-label entry-point)
       ;; FIXME alpha port has a ### note here saying we should "save it
@@ -284,7 +283,7 @@ default-value-8
 
             (let ((defaults (defaults)))
               (when defaults
-                (assemble (*elsewhere*)
+                (assemble (:elsewhere)
                   (emit-label default-stack-vals)
                   (do ((remaining defaults (cdr remaining)))
                       ((null remaining))
@@ -331,7 +330,7 @@ default-value-8
 
     (emit-label done)
 
-    (assemble (*elsewhere*)
+    (assemble (:elsewhere)
       (emit-label variable-values)
       (inst compute-code-from-lra code-tn lra-tn lra-label temp)
       (do ((arg *register-arg-tns* (rest arg))
@@ -1049,7 +1048,7 @@ default-value-8
            (inst beq DONE))
           (t
            (inst addic. count nargs-tn (- (fixnumize fixed)))
-           (inst ble (assemble (*elsewhere*)
+           (inst ble (assemble (:elsewhere)
                        FIX-CSP
                        ;; If the fixed args aren't fully supplied, the
                        ;; (INST ADD CSP-TN RESULT COUNT) below could
