@@ -217,7 +217,11 @@
   `(typep ,x 'base-char))
 ;; CONS is implemented as (and list (not (eql nil))) where the 'and' is
 ;; built-in to the consp vop. Reduce to just LISTP if possible.
-(deftransform consp ((x) ((not null)) * :important nil) '(listp x))
+(deftransform consp ((x) ((not null)) * :important nil)
+  '(listp x))
+
+(deftransform symbolp ((x) ((not null)) * :important nil)
+  '(non-null-symbol-p x))
 
 ;;;; TYPEP source transform
 
@@ -267,30 +271,35 @@
                  (rational 'rational)
                  (float (or (numeric-type-format type) 'float))
                  ((nil) 'real))))
-    (once-only ((n-object object))
-      (ecase (numeric-type-complexp type)
-        (:real
-         (cond #!+(or x86 x86-64 arm arm64) ;; Not implemented elsewhere yet
-               ((and
-                 (eql (numeric-type-class type) 'integer)
-                 (eql (numeric-type-low type) 0)
-                 (fixnump (numeric-type-high type)))
-                `(fixnum-mod-p ,n-object ,(numeric-type-high type)))
-               (t
-                `(and (typep ,n-object ',base)
-                      ,(transform-numeric-bound-test n-object type base)))))
-        (:complex
-         `(and (complexp ,n-object)
-               ,(once-only ((n-real `(realpart (truly-the complex ,n-object)))
-                            (n-imag `(imagpart (truly-the complex ,n-object))))
-                  `(progn
-                     ,n-imag ; ignorable
-                     (and (typep ,n-real ',base)
-                          ,@(when (eq class 'integer)
-                              `((typep ,n-imag ',base)))
-                          ,(transform-numeric-bound-test n-real type base)
-                          ,(transform-numeric-bound-test n-imag type
-                                                         base))))))))))
+    (ecase (numeric-type-complexp type)
+      (:real
+       (cond #!+(or x86 x86-64 arm arm64) ;; Not implemented elsewhere yet
+             ((and
+               (eql (numeric-type-class type) 'integer)
+               (or (eql (numeric-type-low type) 0)
+                   (eql (numeric-type-low type) 1))
+               (fixnump (numeric-type-high type)))
+              (let ((mod-p
+                      `(fixnum-mod-p ,object ,(numeric-type-high type))))
+                (if (eql (numeric-type-low type) 1)
+                    `(and (not (eq ,object 0))
+                          ,mod-p)
+                    mod-p)))
+             (t
+              `(and (typep ,object ',base)
+                    ,(transform-numeric-bound-test object type base)))))
+      (:complex
+       `(and (complexp ,object)
+             ,(once-only ((n-real `(realpart (truly-the complex ,object)))
+                          (n-imag `(imagpart (truly-the complex ,object))))
+                `(progn
+                   ,n-imag ; ignorable
+                   (and (typep ,n-real ',base)
+                        ,@(when (eq class 'integer)
+                            `((typep ,n-imag ',base)))
+                        ,(transform-numeric-bound-test n-real type base)
+                        ,(transform-numeric-bound-test n-imag type
+                                                       base)))))))))
 
 ;;; Do the source transformation for a test of a hairy type. AND,
 ;;; SATISFIES and NOT are converted into the obvious code. We convert
@@ -328,54 +337,161 @@
                           `(funcall (global-function ,name) ,object))
                      t nil)))
              ((not and)
-              (once-only ((n-obj object))
-                `(,(first spec) ,@(mapcar (lambda (x)
-                                            `(typep ,n-obj ',x))
-                                          (rest spec))))))))))
+              `(,(first spec) ,@(mapcar (lambda (x)
+                                          `(typep ,object ',x))
+                                        (rest spec)))))))))
 
 (defun source-transform-negation-typep (object type)
   (declare (type negation-type type))
   (let ((spec (type-specifier (negation-type-type type))))
     `(not (typep ,object ',spec))))
 
+;;; Check the type of a group of equally specialized but of
+;;; different length simple arrays once
+(defun group-vector-type-length-tests (object types)
+  (let (groups
+        any-grouped)
+    (loop for type in types
+          do
+          (if (and (array-type-p type)
+                   (not (array-type-complexp type))
+                   (typep (array-type-dimensions type) '(cons integer null))
+                   (or (eq (array-type-element-type type) *wild-type*)
+                       (neq (array-type-specialized-element-type type) *wild-type*)))
+              (push type
+                    (getf groups
+                          (array-type-specialized-element-type type)))
+              (push type (getf groups :other))))
+    (loop for (el-type types) on groups by #'cddr
+          do
+          (cond ((eq el-type :other))
+                ((> (length types) 1)
+                 (setf any-grouped t))
+                (t
+                 (push (car types)
+                       (getf groups :other)))))
+    (when any-grouped
+      (let ((other (getf groups :other)))
+        `(or
+          ,@(loop for (el-type types) on groups by #'cddr
+                  when (and (neq el-type :other)
+                            (> (length types) 1))
+                  collect `(and (typep ,object
+                                       '(simple-array ,(type-specifier el-type) (*)))
+                                (typep (vector-length
+                                        (truly-the (simple-array * (*)) ,object))
+                                       '(member ,@(loop for type in types
+                                                        collect (car (array-type-dimensions type)))))))
+          ,@(and
+             other
+             `((typep ,object '(or ,@(mapcar #'type-specifier other))))))))))
+
+;;; Test the length of multiple arrays types once
+(defun group-vector-length-type-tests (object types)
+  (let (groups
+        any-grouped)
+    (loop for type in types
+          do
+          (if (and (array-type-p type)
+                   (typep (array-type-dimensions type) '(cons integer null)))
+              (push type (getf groups (car (array-type-dimensions type))))
+              (push type (getf groups :other))))
+    (loop for (length types) on groups by #'cddr
+          do
+          (cond ((eq length :other))
+                ((> (length types) 1)
+                 (setf any-grouped t))
+                (t
+                 (push (car types)
+                       (getf groups :other)))))
+    (when any-grouped
+      (let ((other (getf groups :other)))
+        `(or
+          ,@(loop for (length types) on groups by #'cddr
+                  for any-complex = nil
+                  for any-simple = nil
+                  when (and (neq length :other)
+                            (> (length types) 1))
+                  collect `(and (typep ,object
+                                       '(or
+                                         ,@(loop for type in types
+                                                 for complex = (array-type-complexp type)
+                                                 do (cond (complex
+                                                           (setf any-complex t)
+                                                           (when (eq complex :maybe)
+                                                             (setf any-simple t)))
+                                                          (t
+                                                           (setf any-simple t)))
+
+                                                 collect
+                                                 (type-specifier
+                                                  (make-array-type '(*)
+                                                                   :complexp complex
+                                                                   :element-type
+                                                                   (array-type-element-type type)
+                                                                   :specialized-element-type
+                                                                   (array-type-specialized-element-type type))))))
+                                ,(cond
+                                   ((not any-complex)
+                                    `(= (vector-length (truly-the (simple-array * (*)) ,object))
+                                        ,length))
+                                   ((not any-simple)
+                                    `(= (%array-dimension (truly-the vector ,object) 0)
+                                        ,length))
+                                   (t
+                                    `(if (array-header-p (truly-the vector ,object))
+                                         (= (%array-dimension (truly-the vector ,object) 0)
+                                            ,length)
+                                         (= (vector-length (truly-the vector ,object))
+                                            ,length))))))
+          ,@(and
+             other
+             `((typep ,object '(or ,@(mapcar #'type-specifier other))))))))))
+
 ;;; Do source transformation for TYPEP of a known union type. If a
 ;;; union type contains LIST, then we pull that out and make it into a
-;;; single LISTP call.  Note that if SYMBOL is in the union, then LIST
-;;; will be a subtype even without there being any (member NIL).  We
-;;; currently just drop through to the general code in this case,
-;;; rather than trying to optimize it (but FIXME CSR 2004-04-05: it
-;;; wouldn't be hard to optimize it after all).
-;;; FIXME: if the CONSP|NIL -> LISTP optimization kicks in,
-;;;        we forgo the array optimizations.
+;;; single LISTP call.
 (defun source-transform-union-typep (object type)
   (let* ((types (union-type-types type))
          (type-cons (specifier-type 'cons))
+         (type-symbol (specifier-type 'symbol))
          (mtype (find-if #'member-type-p types))
          (members (when mtype (member-type-members mtype))))
-    (once-only ((n-obj object))
-      (if (and mtype
-               (memq nil members)
-               (memq type-cons types))
-          `(or (listp ,n-obj)
-               (typep ,n-obj
-                      '(or ,@(mapcar #'type-specifier
-                                     (remove type-cons
-                                             (remove mtype types)))
-                        (member ,@(remove nil members)))))
-          (multiple-value-bind (widetags more-types)
-              (sb!kernel::widetags-from-union-type types)
-            `(or ,@(if widetags
-                       `((%other-pointer-subtype-p ,n-obj ',widetags)))
-                 ,@(mapcar (lambda (x)
-                             `(typep ,n-obj ',(type-specifier x)))
-                           more-types)))))))
+    (cond ((and mtype
+                (memq nil members)
+                (memq type-cons types))
+           `(or (listp ,object)
+                (typep ,object
+                       '(or ,@(mapcar #'type-specifier
+                               (remove type-cons
+                                (remove mtype types)))
+                         (member ,@(remove nil members))))))
+          ((and (memq type-cons types)
+                (memq type-symbol types))
+           `(or (listp ,object)
+                (non-null-symbol-p ,object)
+                (typep ,object
+                       '(or ,@(mapcar #'type-specifier
+                               (remove type-cons
+                                (remove type-symbol types)))))))
+          ((group-vector-type-length-tests object types))
+          ((group-vector-length-type-tests object types))
+          (t
+           (multiple-value-bind (widetags more-types)
+               (sb!kernel::widetags-from-union-type types)
+             (if widetags
+                 `(or (%other-pointer-subtype-p ,object ',widetags)
+                      (typep ,object '(or ,@(mapcar #'type-specifier more-types))))
+                 `(or
+                   ,@(mapcar (lambda (x)
+                               `(typep ,object ',(type-specifier x)))
+                             more-types))))))))
 
 ;;; Do source transformation for TYPEP of a known intersection type.
 (defun source-transform-intersection-typep (object type)
-  (once-only ((n-obj object))
-    `(and ,@(mapcar (lambda (x)
-                      `(typep ,n-obj ',(type-specifier x)))
-                    (intersection-type-types type)))))
+  `(and ,@(mapcar (lambda (x)
+                    `(typep ,object ',(type-specifier x)))
+                  (intersection-type-types type))))
 
 ;;; If necessary recurse to check the cons type.
 (defun source-transform-cons-typep (object type)
@@ -409,23 +525,21 @@
                       ;; because NIL is not a keyword.
                       (equal (hairy-type-specifier ctype)
                              '(satisfies keywordp))))))
-          (let* ((n-obj (sb!xc:gensym))
-                 (car-test
+          (let ((car-test
                   (and car-test-p
-                       `((typep (car ,n-obj) ',(type-specifier car-type)))))
-                 (cdr-test
+                       `((typep (car ,object) ',(type-specifier car-type)))))
+                (cdr-test
                   (and cdr-test-p
-                       `((typep (cdr ,n-obj) ',(type-specifier cdr-type))))))
-            `(let ((,n-obj ,object))
-               ;; Being paranoid, perform the safely weakenable test first
-               ;; so that the other part doesn't execute on an object that
-               ;; it would not have gotten, were the CONSP test not weakened.
-               ,(cond ((and car-test-p (safely-weakened car-type))
-                       `(and (listp ,n-obj) ,@car-test ,@cdr-test))
-                      ((and cdr-test-p (safely-weakened cdr-type))
-                       `(and (listp ,n-obj) ,@cdr-test ,@car-test))
-                      (t
-                       `(and (consp ,n-obj) ,@car-test ,@cdr-test)))))))))
+                       `((typep (cdr ,object) ',(type-specifier cdr-type))))))
+            ;; Being paranoid, perform the safely weakenable test first
+            ;; so that the other part doesn't execute on an object that
+            ;; it would not have gotten, were the CONSP test not weakened.
+            (cond ((and car-test-p (safely-weakened car-type))
+                   `(and (listp ,object) ,@car-test ,@cdr-test))
+                  ((and cdr-test-p (safely-weakened cdr-type))
+                   `(and (listp ,object) ,@cdr-test ,@car-test))
+                  (t
+                   `(and (consp ,object) ,@car-test ,@cdr-test))))))))
 
 (defun source-transform-character-set-typep (object type)
   (let ((pairs (character-set-type-pairs type)))
@@ -437,28 +551,26 @@
                 `(base-char-p ,object))
                ((= (cdar pairs) (1- sb!xc:char-code-limit))
                 `(characterp ,object))))
-        (once-only ((n-obj object))
-          (let ((n-code (gensym "CODE")))
-            `(and (characterp ,n-obj)
-                  (let ((,n-code (sb!xc:char-code ,n-obj)))
-                    (or
-                     ,@(loop for pair in pairs
-                             collect
-                             `(<= ,(car pair) ,n-code ,(cdr pair)))))))))))
+        (let ((n-code (gensym "CODE")))
+          `(and (characterp ,object)
+                (let ((,n-code (sb!xc:char-code ,object)))
+                  (or
+                   ,@(loop for pair in pairs
+                           collect
+                           `(<= ,(car pair) ,n-code ,(cdr pair))))))))))
 
 #!+sb-simd-pack
 (defun source-transform-simd-pack-typep (object type)
   (if (type= type (specifier-type 'simd-pack))
       `(simd-pack-p ,object)
-      (once-only ((n-obj object))
-        (let ((n-tag (gensym "TAG")))
-          `(and
-            (simd-pack-p ,n-obj)
-            (let ((,n-tag (%simd-pack-tag ,n-obj)))
-              (or ,@(loop
-                      for type in (simd-pack-type-element-type type)
-                      for index = (position type *simd-pack-element-types*)
-                      collect `(eql ,n-tag ,index)))))))))
+      (let ((n-tag (gensym "TAG")))
+        `(and
+          (simd-pack-p ,object)
+          (let ((,n-tag (%simd-pack-tag ,object)))
+            (or ,@(loop
+                    for type in (simd-pack-type-element-type type)
+                    for index = (position type *simd-pack-element-types*)
+                    collect `(eql ,n-tag ,index))))))))
 
 ;;; Return the predicate and type from the most specific entry in
 ;;; *TYPE-PREDICATES* that is a supertype of TYPE.
@@ -481,23 +593,30 @@
 ;;;
 ;;; Secondary return value is true if passing the generated tests implies that
 ;;; the array has a header.
-(defun test-array-dimensions (obj type stype)
+(defun test-array-dimensions (obj type stype
+                              simple-array-header-p)
   (declare (type array-type type stype))
   (let ((obj `(truly-the ,(type-specifier stype) ,obj))
-        (dims (array-type-dimensions type)))
+        (dims (array-type-dimensions type))
+        (header-test (if simple-array-header-p
+                         `(simple-array-header-p ,obj)
+                         `(array-header-p ,obj))))
     (unless (or (eq dims '*)
                 (equal dims (array-type-dimensions stype)))
       (cond ((cdr dims)
-             (values `((array-header-p ,obj)
+             (values `(,header-test
                        ,@(when (eq (array-type-dimensions stype) '*)
-                               `((= (%array-rank ,obj) ,(length dims))))
+                           #!+x86-64
+                           `((%array-rank= ,obj ,(length dims)))
+                           #!-x86-64
+                           `((= (%array-rank ,obj) ,(length dims))))
                        ,@(loop for d in dims
                                for i from 0
                                unless (eq '* d)
                                collect `(= (%array-dimension ,obj ,i) ,d)))
                      t))
             ((not dims)
-             (values `((array-header-p ,obj)
+             (values `(,header-test
                        (= (%array-rank ,obj) 0))
                      t))
             ((not (array-type-complexp type))
@@ -506,13 +625,13 @@
                            `((= (vector-length ,obj) ,@dims)))
                          nil)
                  (values (if (eq '* (car dims))
-                             `((not (array-header-p ,obj)))
-                             `((not (array-header-p ,obj))
+                             `((not ,header-test))
+                             `((not ,header-test)
                                (= (vector-length ,obj) ,@dims)))
                          nil)))
             (t
              (values (unless (eq '* (car dims))
-                       `((if (array-header-p ,obj)
+                       `((if ,header-test
                              (= (%array-dimension ,obj 0) ,@dims)
                              (= (vector-length ,obj) ,@dims))))
                      nil))))))
@@ -533,7 +652,10 @@
              ;; If we know OBJ is an array header, and that the array is
              ;; simple, we also know there is exactly one indirection to
              ;; follow.
-             `((eq (%other-pointer-widetag (%array-data ,obj)) ,typecode))
+             `(#!-x86-64
+               (eq (%other-pointer-widetag (%array-data ,obj)) ,typecode)
+               #!+x86-64
+               (widetag= (%array-data ,obj) ,typecode))
              `((do ((,data ,(if headerp `(%array-data ,obj) obj)
                            (%array-data ,data)))
                    ((not (array-header-p ,data))
@@ -542,7 +664,7 @@
 ;;; If we can find a type predicate that tests for the type without
 ;;; dimensions, then use that predicate and test for dimensions.
 ;;; Otherwise, just do %TYPEP.
-(defun source-transform-array-typep (obj type)
+(defun source-transform-array-typep (object type)
   ;; Intercept (SIMPLE-ARRAY * (*)) because otherwise it tests
   ;; (AND SIMPLE-ARRAY (NOT ARRAY-HEADER)) to weed out rank 0 and >1.
   ;; By design the simple arrays of of rank 1 occupy a contiguous
@@ -550,47 +672,52 @@
   ;; this nonstandard predicate can be generically defined for all backends.
   (let ((dims (array-type-dimensions type))
         (et (array-type-element-type type)))
-   (if (and (not (array-type-complexp type))
-            (eq et *wild-type*)
-            (equal dims '(*)))
-       (return-from source-transform-array-typep
-         `(simple-rank-1-array-*-p ,obj)))
-    (multiple-value-bind (pred stype) (find-supertype-predicate type)
-      (if (and (array-type-p stype)
-               ;; (If the element type hasn't been defined yet, it's
-               ;; not safe to assume here that it will eventually
-               ;; have (UPGRADED-ARRAY-ELEMENT-TYPE type)=T, so punt.)
-               (not (unknown-type-p (array-type-element-type type)))
-               (or (eq (array-type-complexp stype) (array-type-complexp type))
-                   (and (eql (array-type-complexp stype) :maybe)
-                        (eql (array-type-complexp type) t))))
-          (let ((complex-tag (and
-                              (eql (array-type-complexp type) t)
-                              (singleton-p dims)
-                              (and (neq et *wild-type*)
-                                   (sb!vm:saetp-complex-typecode
-                                    (find-saetp-by-ctype (array-type-element-type type)))))))
-            (once-only ((n-obj obj))
-              (if complex-tag
-                  `(and (eq (%other-pointer-widetag ,n-obj) ,complex-tag)
-                        ,@(unless (eq (car dims) '*)
-                            `((= (%array-dimension ,n-obj 0) ,(car dims)))))
-                  (multiple-value-bind (tests headerp)
-                      (test-array-dimensions n-obj type stype)
-                    `(and ,@(unless (and headerp (eql pred 'arrayp))
-                              ;; ARRAY-HEADER-P from TESTS will test for that
-                              `((,pred ,n-obj)))
-                          ,@(when (and (eql (array-type-complexp stype) :maybe)
-                                       (eql (array-type-complexp type) t))
-                              ;; KLUDGE: this is a bit lame; if we get here,
-                              ;; we already know that N-OBJ is an array, but
-                              ;; (NOT SIMPLE-ARRAY) doesn't know that.  On the
-                              ;; other hand, this should get compiled down to
-                              ;; two widetag tests, so it's only a bit lame.
-                              `((typep ,n-obj '(not simple-array))))
-                          ,@tests
-                          ,@(test-array-element-type n-obj type stype headerp))))))
-          `(%typep ,obj ',(type-specifier type))))))
+    (if (and (not (array-type-complexp type))
+             (eq et *wild-type*)
+             (equal dims '(*)))
+        `(simple-rank-1-array-*-p ,object)
+        (multiple-value-bind (pred stype) (find-supertype-predicate type)
+          (if (and (array-type-p stype)
+                   ;; (If the element type hasn't been defined yet, it's
+                   ;; not safe to assume here that it will eventually
+                   ;; have (UPGRADED-ARRAY-ELEMENT-TYPE type)=T, so punt.)
+                   (not (unknown-type-p (array-type-element-type type)))
+                   (or (eq (array-type-complexp stype) (array-type-complexp type))
+                       (and (eql (array-type-complexp stype) :maybe)
+                            (eql (array-type-complexp type) t))))
+              (let ((complex-tag (and
+                                  (eql (array-type-complexp type) t)
+                                  (singleton-p dims)
+                                  (and (neq et *wild-type*)
+                                       (sb!vm:saetp-complex-typecode
+                                        (find-saetp-by-ctype (array-type-element-type type))))))
+                    (simple-array-header-p
+                      (and (null (array-type-complexp stype))
+                           (listp dims)
+                           (cdr dims))))
+                (if complex-tag
+                    `(and (%other-pointer-p ,object)
+                          (eq (%other-pointer-widetag ,object) ,complex-tag)
+                          ,@(unless (eq (car dims) '*)
+                              `((= (%array-dimension ,object 0) ,(car dims)))))
+                    (multiple-value-bind (tests headerp)
+                        (test-array-dimensions object type stype
+                                               simple-array-header-p)
+                      `(and ,@(unless (or (and headerp (eql pred 'arrayp))
+                                          simple-array-header-p)
+                                ;; ARRAY-HEADER-P from TESTS will test for that
+                                `((,pred ,object)))
+                            ,@(when (and (eql (array-type-complexp stype) :maybe)
+                                         (eql (array-type-complexp type) t))
+                                ;; KLUDGE: this is a bit lame; if we get here,
+                                ;; we already know that OBJECT is an array, but
+                                ;; (NOT SIMPLE-ARRAY) doesn't know that.  On the
+                                ;; other hand, this should get compiled down to
+                                ;; two widetag tests, so it's only a bit lame.
+                                `((typep ,object '(not simple-array))))
+                            ,@tests
+                            ,@(test-array-element-type object type stype headerp)))))
+              `(%typep ,object ',(type-specifier type)))))))
 
 ;;; Transform a type test against some instance type. The type test is
 ;;; flushed if the result is known at compile time. If not properly
@@ -605,11 +732,7 @@
   (let* ((spec (lvar-value spec))
          (class (specifier-type spec))
          (name (classoid-name class))
-         (otype (lvar-type object))
-         (layout (let ((res (info :type :compiler-layout name)))
-                   (if (and res (not (layout-invalid res)))
-                       res
-                       nil))))
+         (otype (lvar-type object)))
     (cond
       ;; Flush tests whose result is known at compile time.
       ((not (types-equal-or-intersect otype class))
@@ -624,7 +747,14 @@
       (t
        ;; Delay the type transform to give type propagation a chance.
        (delay-ir1-transform node :constraint)
+       (transform-instance-typep class)))))
 
+(defun transform-instance-typep (class)
+  (let* ((name (classoid-name class))
+          (layout (let ((res (info :type :compiler-layout name)))
+                   (if (and res (not (layout-invalid res)))
+                       res
+                       nil))))
        ;; FIXME: (TYPEP X 'ERROR) - or any condition - checks whether X
        ;; has the lowtag of either an ordinary or funcallable instance.
        ;; But you can not define a class that is both CONDITION and FUNCTION
@@ -675,13 +805,19 @@
                                    (declare (ignore classoid))
                                    (return layout)))))
               (flet ((check-layout (layout-getter)
-                       (if other-layout
-                           ;; It's faster to compare two layouts than
-                           ;; doing whatever is done below
-                           `(let ((object-layout ,layout-getter))
-                              (or (eq object-layout ',layout)
-                                  (eq object-layout ',other-layout)))
-                           `(eq ,layout-getter ',layout))))
+                       (cond (other-layout
+                              ;; It's faster to compare two layouts than
+                              ;; doing whatever is done below
+                              `(let ((object-layout ,layout-getter))
+                                 (or (eq object-layout ',layout)
+                                     (eq object-layout ',other-layout))))
+                             ;; FIXME: not defined in time for make-host-1,
+                             ;; so the cross-compiler isn't getting this branch.
+                             #!+(vop-named sb!vm::layout-eq)
+                             ((equal layout-getter '(%instance-layout object))
+                              `(sb!vm::layout-eq object ',layout))
+                             (t
+                              `(eq ,layout-getter ',layout)))))
                 (if pred
                     `(and ,pred ,(check-layout get-layout))
                     `(block typep ,(check-layout get-layout-or-return-false))))))
@@ -763,7 +899,7 @@
 
            (t
             `(classoid-cell-typep ',(find-classoid-cell name :create t)
-                                  object))))))))
+                                  object))))))
 
 ;;; If the specifier argument is a quoted constant, then we consider
 ;;; converting into a simple predicate or other stuff. If the type is
@@ -786,9 +922,18 @@
               (multiple-value-bind (constantp value) (type-singleton-p ctype)
                 (and constantp
                      `(eql ,object ',value))))
-         (let ((pred (cdr (assoc ctype *backend-type-predicates*
-                                 :test #'type=))))
-           (when pred `(,pred ,object)))
+         (handler-case
+             (or
+              (let ((pred (cdr (assoc ctype *backend-type-predicates*
+                                      :test #'type=))))
+                (when pred `(,pred ,object)))
+              (let ((pred (cdr (assoc (type-negation ctype)
+                                      *backend-type-predicates*
+                                      :test #'type=))))
+                (when pred `(not (,pred ,object)))))
+           #+sb-xc-host
+           (sb!kernel::cross-type-warning
+             nil))
          (typecase ctype
            (hairy-type
             (source-transform-hairy-typep object ctype))
@@ -1019,3 +1164,16 @@
 (deftransform #!+64-bit unsigned-byte-64-p #!-64-bit unsigned-byte-32-p
   ((value) (fixnum))
   `(>= value 0))
+
+(deftransform %other-pointer-p ((object))
+  (let ((this-type
+          (specifier-type '(or fixnum
+                            #!+64-bit single-float
+                            function
+                            list
+                            instance
+                            character))))
+    (cond ((not (types-equal-or-intersect this-type (lvar-type object))))
+          ((csubtypep (lvar-type object) this-type)
+           nil)
+          ((give-up-ir1-transform)))))
