@@ -237,7 +237,7 @@
   (define-binop logior 2 or
     :c/unsigned=>unsigned
     ((let ((y (constantize y)))
-       (cond ((and (register-p r) (eql y -1)) ; special-case "OR reg, all-ones"
+       (cond ((and (gpr-tn-p r) (eql y -1)) ; special-case "OR reg, all-ones"
               ;; I have yet to elicit this case. Can it happen?
               (inst mov r -1))
              (t
@@ -1238,14 +1238,14 @@ constant shift greater than word length")))
                    (inst mov :dword temp-reg-tn y)
                    (return-from ensure-not-mem+mem (values x temp-reg-tn))))
            (setq y (register-inline-constant :qword y)))
-         (cond ((or (gpr-p x) (gpr-p y))
+         (cond ((or (gpr-tn-p x) (gpr-tn-p y))
                 (values x y))
                (t
                 (inst mov temp-reg-tn x)
                 (values temp-reg-tn y))))))
 
 (defun immediate-operand-smallest-nbits (x)
-  (declare (word x))
+  (declare (type word x))
   (typecase x
     ((unsigned-byte  8)  8)
     ((unsigned-byte 16) 16)
@@ -1296,9 +1296,9 @@ constant shift greater than word length")))
                (setq size :byte disp (1+ disp) y (ash y -8)))
              (inst test size (ea disp rbp-tn) y)))
           (t
-           (aver (gpr-p x))
-           (if (and reducible-to-byte-p (<= (tn-offset x) 6)) ; 0, 2, 4, 6
-               ;; Use upper byte of word reg (AX -> AH, BX -> BX ...)
+           (aver (gpr-tn-p x))
+           (if (and reducible-to-byte-p (<= (tn-offset x) rbx-offset))
+               ;; Use upper byte of word reg {A,C,D,B}X -> {A,C,D,B}H
                (inst test :byte `(,x . :high-byte) (ash y -8))
                (inst test size x y))))))
 
@@ -1387,7 +1387,7 @@ constant shift greater than word length")))
     (inst bt x y)))
 
 (defun emit-optimized-cmp (x y)
-  (if (and (gpr-p x) (eql y 0))
+  (if (and (gpr-tn-p x) (eql y 0))
       ;; Amazingly (to me), use of TEST in lieu of CMP produces all the correct
       ;; flag bits for inequality comparison as well as EQL comparison.
       ;; You'd think that the Jxx instruction should examine _only_ the S flag,
