@@ -21,11 +21,12 @@
   (generate-fixnum-test value)
   (inst jmp (if not-p :nz :z) target))
 
-(defun %test-fixnum-and-headers (value temp target not-p headers)
+(defun %test-fixnum-and-headers (value temp target not-p headers &key value-tn-ref)
   (let ((drop-through (gen-label)))
     (generate-fixnum-test value)
     (inst jmp :z (if not-p drop-through target))
-    (%test-headers value temp target not-p nil headers :drop-through drop-through)))
+    (%test-headers value temp target not-p nil headers
+                   :drop-through drop-through :value-tn-ref value-tn-ref)))
 
 (defun %test-immediate (value temp target not-p immediate)
   (declare (ignore temp))
@@ -58,7 +59,7 @@
   (inst jmp (if not-p :ne :e) target))
 
 (defun %test-headers (value temp target not-p function-p headers
-                            &key except (drop-through (gen-label)))
+                      &key except (drop-through (gen-label)) value-tn-ref)
   (let ((lowtag (if function-p fun-pointer-lowtag other-pointer-lowtag)))
     (multiple-value-bind (equal less-or-equal greater-or-equal when-true when-false)
         ;; EQUAL, LESS-OR-EQUAL and GREATER-OR-EQUAL are the conditions for
@@ -68,7 +69,10 @@
         (if not-p
             (values :ne :a :b drop-through target)
             (values :e :na :nb target drop-through))
-      (%test-lowtag value temp when-false t lowtag)
+      (unless (and value-tn-ref
+                   (eq lowtag other-pointer-lowtag)
+                   (other-pointer-tn-ref-p value-tn-ref))
+        (%test-lowtag value temp when-false t lowtag))
       (cond
         ((and (null (cdr headers))
               (not except)
@@ -149,9 +153,7 @@
                         (inst cmp al-tn start)
                         (inst jmp :b when-false)
                         (inst cmp al-tn end)
-                        (if last
-                            (inst jmp less-or-equal target)
-                            (inst jmp :be when-true)))
+                        (inst jmp :be when-true))
                        (t
                         (inst sub al-tn start)
                         (inst cmp al-tn (- end start))
@@ -173,7 +175,7 @@
   (:conditional :be)
   (:arg-types unsigned-num)
   (:translate fixnump)
-  (:generator 5
+  (:generator 3
     ;; We could encode this with :Z and SHR, analogously to the signed-byte-32
     ;; case below -- as we do on x86-64 -- but that costs us an extra
     ;; register. Compromises...
@@ -186,7 +188,7 @@
   (:arg-types signed-num)
   (:translate fixnump)
   (:ignore temp)
-  (:generator 5
+  (:generator 3
     ;; Hackers Delight, p. 53: signed
     ;;    a <= x <= a + 2^n - 1
     ;; is equivalent to unsigned
