@@ -12,7 +12,9 @@
 #ifndef _SEARCH_H_
 #define _SEARCH_H_
 
-extern boolean search_for_type(int type, lispobj **start, int *count);
-extern lispobj* search_for_symbol(char *name, lispobj start, lispobj end);
+extern lispobj* find_symbol(char*, char*, unsigned int*); // Find via package
+extern struct symbol* lisp_symbol_from_tls_index(lispobj tls_index);
+// Find via heap scan
+extern lispobj* search_for_symbol(char *name, lispobj start, lispobj end, boolean);
 
 #endif
