@@ -8,22 +8,11 @@
 #define ARCH_HAS_STACK_POINTER
 #define ALIEN_STACK_GROWS_DOWNWARD
 
-static inline lispobj
-swap_lispobjs(volatile lispobj *dest, lispobj value)
-{
-    lispobj old_value;
-    asm volatile
-        ("lock xchg %0,(%1)"
-         : "=r" (old_value)
-         : "r" (dest), "0" (value)
-         : "memory");
-    return old_value;
-}
-
 extern unsigned int avx_supported;
-
+extern unsigned int cpuid_fn1_ecx;
 /* When single stepping, single_stepping holds the original instruction
  * PC location. */
 extern unsigned int *single_stepping;
 
+
 #endif /* _X86_64_ARCH_H */
